import collections


def validate_all_migrations_have_ids(migration_classes):
    migration_ids = [m.migration_id for m in migration_classes]

    if not all(migration_ids):
        raise ValueError("All migrations must have an id")


def validate_migration_id_uniqueness(migration_classes):
    id_counts = collections.Counter((m.migration_id for m in migration_classes))

    duplicate_ids = [
        migration_id for migration_id, count in id_counts.items() if count > 1
    ]
    if duplicate_ids:
        raise ValueError("All migrations must have unique ids")


def validate_no_self_dependencies(migration_classes):
    migration_dependency_graph = {
        m.migration_id: set() if m.dependencies is None else set(m.dependencies)
        for m in migration_classes
    }

    has_self_dependencies = [
        migration_id
        for migration_id, migration_dependencies
        in migration_dependency_graph.items()
        if migration_id in migration_dependencies
    ]

    if has_self_dependencies:
        raise ValueError("Migrations cannot depend on themselves")


def validate_migration_classes(migration_classes):
    validate_all_migrations_have_ids(migration_classes)
    validate_migration_id_uniqueness(migration_classes)
    validate_no_self_dependencies(migration_classes)



import itertools
from collections import OrderedDict

from populus.utils.contracts import (
    get_shallow_dependency_graph,
    get_contract_deploy_order,
    get_recursive_contract_dependencies,
    link_bytecode,
)


def get_deploy_order(contracts_to_deploy, compiled_contracts):
    # Extract and dependencies that exist due to library linking.
    dependency_graph = get_shallow_dependency_graph(compiled_contracts)

    # Compute the full set of dependencies needed to deploy the desired
    # contracts.
    all_deploy_dependencies = set(itertools.chain.from_iterable(
        get_recursive_contract_dependencies(contract_name, dependency_graph)
        for contract_name in contracts_to_deploy
    ))
    all_contracts_to_deploy = all_deploy_dependencies.union(contracts_to_deploy)

    # Now compute the order that the contracts should be deployed based on
    # their dependencies.
    deploy_order = [
        (contract_name, compiled_contracts[contract_name])
        for contract_name
        in get_contract_deploy_order(dependency_graph)
        if contract_name in all_contracts_to_deploy
    ]
    return OrderedDict(deploy_order)


def deploy_contract(chain,
                    contract_name,
                    contract_factory=None,
                    deploy_transaction=None,
                    deploy_arguments=None,
                    link_dependencies=None):
    if contract_factory is None:
        contract_factory = chain.contract_factories[contract_name]

    web3 = chain.web3

    code = link_bytecode(contract_factory.code, **link_dependencies)
    code_runtime = link_bytecode(contract_factory.code_runtime, **link_dependencies)

    ContractFactory = web3.eth.contract(
        abi=contract_factory.abi,
        code=code,
        code_runtime=code_runtime,
        source=contract_factory.source,
    )

    deploy_transaction_hash = ContractFactory.deploy(
        deploy_transaction,
        deploy_arguments,
    )
    return deploy_transaction_hash, ContractFactory



import itertools

try:
    from contextlib import ExitStack
except ImportError:
    from contextlib2 import ExitStack

from testrpc import testrpc

from web3.utils.types import is_string

from web3.providers.rpc import TestRPCProvider
from web3 import (
    Web3,
    RPCProvider,
    IPCProvider,
)

from geth import (
    DevGethProcess,
    LiveGethProcess,
    TestnetGethProcess,
    LoggingMixin,
)
from populus.utils.functional import (
    cached_property,
)
from populus.utils.networking import (
    get_open_port,
    wait_for_connection,
)
from populus.utils.module_loading import (
    import_string,
)
from populus.utils.transactions import (
    wait_for_transaction_receipt,
    get_contract_address_from_txn,
)
from populus.utils.filesystem import (
    remove_file_if_exists,
    remove_dir_if_exists,
    get_blockchains_dir,
    tempdir,
)
from populus.utils.contracts import (
    construct_contract_factories,
    package_contracts,
    get_contract_library_dependencies,
    link_bytecode,
)
from populus.utils.chains import (
    get_chaindata_dir,
    get_dapp_dir,
    get_nodekey_path,
    get_geth_ipc_path,
    get_geth_logfile_path,
)

from populus.migrations.registrar import (
    get_compiled_registrar_contract,
)


TESTNET_BLOCK_1_HASH = '0xad47413137a753b2061ad9b484bf7b0fc061f654b951b562218e9f66505be6ce'
MAINNET_BLOCK_1_HASH = '0x88e96d4537bea4d9c05d12549907b32561d3bf31f45aae734cdc119f13406cb6'


def reset_chain(data_dir):
    chaindata_dir = get_chaindata_dir(data_dir)
    remove_dir_if_exists(chaindata_dir)

    dapp_dir = get_dapp_dir(data_dir)
    remove_dir_if_exists(dapp_dir)

    nodekey_path = get_nodekey_path(data_dir)
    remove_file_if_exists(nodekey_path)

    geth_ipc_path = get_geth_ipc_path(data_dir)
    remove_file_if_exists(geth_ipc_path)


class LoggedDevGethProcess(LoggingMixin, DevGethProcess):
    def __init__(self, project_dir, blockchains_dir, chain_name, overrides):
        stdout_logfile_path = get_geth_logfile_path(
            project_dir,
            chain_name,
            'stdout'
        )
        stderr_logfile_path = get_geth_logfile_path(
            project_dir,
            chain_name,
            'stderr',
        )
        super(LoggedDevGethProcess, self).__init__(
            overrides=overrides,
            chain_name=chain_name,
            base_dir=blockchains_dir,
            stdout_logfile_path=stdout_logfile_path,
            stderr_logfile_path=stderr_logfile_path,
        )


class LoggedMordenGethProccess(LoggingMixin, TestnetGethProcess):
    def __init__(self, project_dir, geth_kwargs):
        super(LoggedMordenGethProccess, self).__init__(
            geth_kwargs=geth_kwargs,
        )


class LoggedMainnetGethProcess(LoggingMixin, LiveGethProcess):
    def __init__(self, project_dir, geth_kwargs):
        super(LoggedMainnetGethProcess, self).__init__(
            geth_kwargs=geth_kwargs,
            stdout_logfile_path=get_geth_logfile_path(
                project_dir,
                'mainnet',
                'stdout'
            ),
            stderr_logfile_path=get_geth_logfile_path(
                project_dir,
                'mainnet',
                'stderr',
            ),
        )


class ContractError(Exception):
    """
    Base Exception class for errors raised by the Chain contract API.
    """
    pass


class BytecodeMismatchError(ContractError):
    """
    Indicates there is a bytecode mismatch.
    """
    pass


class NoKnownAddress(ContractError):
    """
    Raised when the address for a requested contract is not known.
    """
    pass


class UnknownContract(ContractError):
    """
    Raised when the requested contract name is not found in the compiled
    contracts.
    """
    pass


class Chain(object):
    """
    Base class for how populus interacts with the blockchain.

    :param project: Instance of :class:`populus.project.Project`
    """
    project = None
    chain_name = None

    def __init__(self, project, chain_name):
        self.project = project
        self.chain_name = chain_name

    #
    # Required Public API
    #
    @property
    def web3(self):
        raise NotImplementedError("Must be implemented by subclasses")

    @property
    def chain_config(self):
        raise NotImplementedError("Must be implemented by subclasses")

    @cached_property
    def contract_factories(self):
        return construct_contract_factories(
            self.web3,
            self.project.compiled_contracts,
        )

    @property
    def RegistrarFactory(self):
        return get_compiled_registrar_contract(self.web3)

    @property
    def has_registrar(self):
        raise NotImplementedError("Must be implemented by subclasses")

    @property
    def registrar(self):
        raise NotImplementedError("Must be implemented by subclasses")

    def __enter__(self):
        raise NotImplementedError("Must be implemented by subclasses")

    def __exit__(self, exc_type, exc_val, exc_tb):
        pass

    #
    # Utility
    #
    def _extract_library_dependencies(self, bytecode, link_dependencies=None):
        if link_dependencies is None:
            link_dependencies = {}

        all_known_contract_names = set(link_dependencies.keys()).union(
            self.contract_factories.keys(),
        )

        library_dependencies = get_contract_library_dependencies(
            bytecode,
            all_known_contract_names,
        )
        return library_dependencies

    def _link_code(self, bytecodes,
                   link_dependencies=None,
                   validate_bytecode=True,
                   raise_on_error=False):
        if link_dependencies is None:
            link_dependencies = {}

        library_dependencies = set(itertools.chain.from_iterable(
            self._extract_library_dependencies(bytecode, link_dependencies)
            for bytecode in bytecodes
        ))

        # Determine which dependencies would need to be present in the
        # registrar since they were not explicitely declared via the
        # `link_dependencies`.
        registrar_dependencies = set(library_dependencies).difference(
            link_dependencies.keys()
        )
        if registrar_dependencies and not self.has_registrar:
            raise NoKnownAddress(
                "Unable to link bytecode.  Addresses for the following link "
                "dependencies were not provided and this chain does not have a "
                "registrar.\n\n{0}".format(
                    ', '.join(registrar_dependencies),
                )
            )

        # Loop over all of the dependencies and ensure that they are available,
        # raising an exception if they are not.
        for library_name in registrar_dependencies:
            self.is_contract_available(
                library_name,
                link_dependencies=link_dependencies,
                validate_bytecode=validate_bytecode,
                raise_on_error=True,
            )

        registrar_link_dependencies = {
            library_name: self.get_contract(
                library_name,
                link_dependencies=link_dependencies,
                validate_bytecode=validate_bytecode,
            ).address for library_name in registrar_dependencies
        }

        all_link_dependencies = {
            library_name: library_address
            for library_name, library_address
            in itertools.chain(
                link_dependencies.items(),
                registrar_link_dependencies.items(),
            )
        }

        linked_bytecodes = [
            link_bytecode(bytecode, **all_link_dependencies)
            for bytecode in bytecodes
        ]
        return linked_bytecodes

    #
    # Contract API
    #
    def is_contract_available(self,
                              contract_name,
                              link_dependencies=None,
                              validate_bytecode=True,
                              raise_on_error=False):
        if not self.has_registrar:
            raise NoKnownAddress(
                'The `is_contract_available` API is only usable on chains that '
                'have a registrar contract'
            )
        contract_key = 'contract/{name}'.format(name=contract_name)

        if not self.registrar.call().exists(contract_key):
            if raise_on_error:
                raise NoKnownAddress(
                    "Address for contract '{name}' not found in registrar".format(
                        name=contract_name,
                    )
                )
            return False

        if not validate_bytecode:
            return True

        try:
            contract_factory = self.get_contract_factory(
                contract_name,
                link_dependencies=link_dependencies,
            )
        except (NoKnownAddress, BytecodeMismatchError):
            if raise_on_error:
                raise
            return False

        contract_address = self.registrar.call().getAddress(contract_key)

        chain_bytecode = self.web3.eth.getCode(contract_address)

        is_bytecode_match = chain_bytecode == contract_factory.code_runtime
        if not is_bytecode_match and raise_on_error:
            raise BytecodeMismatchError(
                "Bytecode @ {0} does not match expected contract bytecode.\n\n"
                "expected : '{1}'\n"
                "actual   : '{2}'\n".format(
                    contract_address,
                    contract_factory.code_runtime,
                    chain_bytecode,
                ),
            )
        return is_bytecode_match

    def get_contract(self,
                     contract_name,
                     link_dependencies=None,
                     validate_bytecode=True):
        if contract_name not in self.contract_factories:
            raise UnknownContract(
                "No contract found with the name '{0}'.\n\n"
                "Available contracts are: {1}".format(
                    contract_name,
                    ', '.join((name for name in self.contract_factories.keys())),
                )
            )
        self.is_contract_available(
            contract_name,
            link_dependencies=link_dependencies,
            validate_bytecode=validate_bytecode,
            raise_on_error=True,
        )

        contract_factory = self.get_contract_factory(
            contract_name,
            link_dependencies=link_dependencies,
        )
        contract_key = 'contract/{name}'.format(name=contract_name)
        contract_address = self.registrar.call().getAddress(contract_key)
        contract = contract_factory(address=contract_address)
        return contract

    def get_contract_factory(self,
                             contract_name,
                             link_dependencies=None):
        if contract_name not in self.contract_factories:
            raise UnknownContract(
                "No contract found with the name '{0}'.\n\n"
                "Available contracts are: {1}".format(
                    contract_name,
                    ', '.join((name for name in self.contract_factories.keys())),
                )
            )
        base_contract_factory = self.contract_factories[contract_name]

        if link_dependencies is not False:
            code, code_runtime = self._link_code(
                bytecodes=[
                    base_contract_factory.code,
                    base_contract_factory.code_runtime,
                ],
                link_dependencies=link_dependencies,
            )
        else:
            code, code_runtime = (
                base_contract_factory.code,
                base_contract_factory.code_runtime,
            )

        contract_factory = self.web3.eth.contract(
            code=code,
            code_runtime=code_runtime,
            abi=base_contract_factory.abi,
            source=base_contract_factory.source,
        )
        return contract_factory

    @property
    def deployed_contracts(self):
        contract_classes = {
            contract_name: self.get_contract(contract_name)
            for contract_name in self.contract_factories.keys()
            if self.is_contract_available(contract_name)
        }
        return package_contracts(contract_classes)


class ExternalChain(Chain):
    """
    Chain class to represent an externally running blockchain that is not
    locally managed.  This class only houses a pre-configured web3 instance.
    """
    def __init__(self, project, chain_name, *args, **kwargs):
        super(ExternalChain, self).__init__(project, chain_name)

        provider_import_path = kwargs.pop(
            'provider',
            'web3.providers.ipc.IPCProvider',
        )
        provider_class = import_string(provider_import_path)

        if provider_class == RPCProvider:
            host = kwargs.pop('host', '127.0.0.1')
            # TODO: this integer casting needs to be done downstream in
            # web3.py.
            port = int(kwargs.pop('port', 8545))
            provider = provider_class(host=host, port=port)
        elif provider_class == IPCProvider:
            ipc_path = kwargs.pop('ipc_path', None)
            provider = provider_class(ipc_path=ipc_path)
        else:
            raise NotImplementedError(
                "Only the IPCProvider and RPCProvider provider classes are "
                "currently supported for external chains."
            )

        self._web3 = Web3(provider)

    @property
    def chain_config(self):
        return self.project.config.chains[self.chain_name]

    @property
    def web3(self):
        if 'default_account' in self.chain_config:
            self._web3.eth.defaultAccount = self.chain_config['default_account']
        return self._web3

    def __enter__(self):
        return self

    @property
    def has_registrar(self):
        return 'registrar' in self.chain_config

    @cached_property
    def registrar(self):
        if not self.has_registrar:
            raise KeyError(
                "The configuration for the {0} chain does not include a "
                "registrar.  Please set this value to the address of the "
                "deployed registrar contract.".format(self.chain_name)
            )
        return get_compiled_registrar_contract(
            self.web3,
            address=self.chain_config['registrar'],
        )


class TestRPCChain(Chain):
    provider = None
    port = None

    @cached_property
    def web3(self):
        if self.provider is None or not self._running:
            raise ValueError(
                "TesterChain instances must be running to access the web3 "
                "object."
            )
        _web3 = Web3(self.provider)

        if 'default_account' in self.chain_config:
            _web3.eth.defaultAccount = self.chain_config['default_account']

        return _web3

    @property
    def chain_config(self):
        config = self.project.config.chains[self.chain_name]
        # TODO: how to do this without causing a circular dependency between these properties.
        # config.update({
        #     'registrar': self.registrar.address,
        # })
        return config

    has_registrar = True

    @cached_property
    def registrar(self):
        # deploy the registrar
        deploy_txn_hash = self.RegistrarFactory.deploy()
        registrar_address = get_contract_address_from_txn(
            self.web3,
            deploy_txn_hash,
            1,
        )

        return self.RegistrarFactory(address=registrar_address)

    full_reset = staticmethod(testrpc.full_reset)
    reset = staticmethod(testrpc.evm_reset)

    @staticmethod
    def snapshot(*args, **kwargs):
        return int(testrpc.evm_snapshot(*args, **kwargs), 16)

    revert = staticmethod(testrpc.evm_revert)
    mine = staticmethod(testrpc.evm_mine)

    _running = False

    def __enter__(self):
        if self._running:
            raise ValueError("The TesterChain is already running")

        if self.port is None:
            self.port = get_open_port()

        self.provider = TestRPCProvider(port=self.port)

        testrpc.full_reset()
        testrpc.rpc_configure('eth_mining', False)
        testrpc.rpc_configure('eth_protocolVersion', '0x3f')
        testrpc.rpc_configure('net_version', 1)
        testrpc.evm_mine()

        wait_for_connection('127.0.0.1', self.port)
        self._running = True
        return self

    def __exit__(self, *exc_info):
        if not self._running:
            raise ValueError("The TesterChain is not running")
        try:
            self.provider.server.stop()
            self.provider.server.close()
            self.provider.thread.kill()
        finally:
            self._running = False

    def get_contract(self, contract_name, link_dependencies=None, *args, **kwargs):
        if contract_name not in self.contract_factories:
            raise UnknownContract(
                "No contract found with the name '{0}'.\n\n"
                "Available contracts are: {1}".format(
                    contract_name,
                    ', '.join((name for name in self.contract_factories.keys())),
                )
            )

        registrar = self.registrar
        contract_key = "contract/{name}".format(name=contract_name)
        if not registrar.call().exists(contract_key):
            # First dig down into the dependency tree to make the library
            # dependencies available.
            contract_bytecode = self.contract_factories[contract_name].code
            contract_dependencies = self._extract_library_dependencies(
                contract_bytecode, link_dependencies,
            )
            for dependency in contract_dependencies:
                self.get_contract(
                    dependency,
                    link_dependencies=link_dependencies,
                    *args,
                    **kwargs
                )

            # Then get the factory and deploy it.
            contract_factory = self.get_contract_factory(
                contract_name,
                link_dependencies=kwargs.get('link_dependencies'),
            )
            deploy_txn_hash = contract_factory.deploy()
            contract_address = get_contract_address_from_txn(
                self.web3,
                deploy_txn_hash,
            )

            # Then register the address with the registrar so that the super
            # method will be able to get and return it.
            register_txn_hash = registrar.transact().setAddress(
                contract_key,
                contract_address,
            )
            wait_for_transaction_receipt(self.web3, register_txn_hash)
        return super(TestRPCChain, self).get_contract(
            contract_name,
            link_dependencies=link_dependencies,
            *args,
            **kwargs
        )


GETH_KWARGS = {
    'data_dir',
    'geth_executable',
    'max_peers',
    'network_id',
    'no_discover',
    'mine',
    'autodag',
    'miner_threads',
    'nice',
    'unlock',
    'password',
    'port',
    'verbosity',
    'ipc_disable',
    'ipc_path',
    'ipc_api',
    'ws_enabled',
    'ws_enabled',
    'ws_addr',
    'ws_origins',
    'ws_port',
    'ws_api',
    'rpc_enabled',
    'rpc_addr',
    'rpc_port',
    'rpc_api',
    'prefix_cmd',
    'suffix_args',
    'suffix_kwargs',
}


class BaseGethChain(Chain):
    geth = None
    provider_class = None

    def __init__(self, project, chain_name, provider=IPCProvider, **geth_kwargs):
        super(BaseGethChain, self).__init__(project, chain_name)

        if geth_kwargs is None:
            geth_kwargs = {}

        if is_string(provider):
            provider = import_string(provider)

        # context manager shenanigans
        self.stack = ExitStack()

        self.provider_class = provider
        self.extra_kwargs = {
            key: value
            for key, value in geth_kwargs.items() if key not in GETH_KWARGS
        }
        self.geth_kwargs = {
            key: value
            for key, value in geth_kwargs.items() if key in GETH_KWARGS
        }
        self.geth = self.get_geth_process_instance()

    _web3 = None

    @property
    def web3(self):
        if not self.geth.is_running:
            raise ValueError(
                "Underlying geth process doesn't appear to be running"
            )

        if self._web3 is None:
            if issubclass(self.provider_class, IPCProvider):
                provider = IPCProvider(self.geth.ipc_path)
            elif issubclass(self.provider_class, RPCProvider):
                provider = RPCProvider(port=self.geth.rpc_port)
            else:
                raise NotImplementedError(
                    "Unsupported provider class {0!r}.  Must be one of "
                    "IPCProvider or RPCProvider"
                )
            _web3 = Web3(provider)

            if 'default_account' in self.chain_config:
                _web3.eth.defaultAccount = self.chain_config['default_account']

            self._web3 = _web3
        return self._web3

    @property
    def chain_config(self):
        return self.project.config.chains[self.chain_name]

    @property
    def has_registrar(self):
        return 'registrar' in self.chain_config

    @cached_property
    def registrar(self):
        if not self.has_registrar:
            raise KeyError(
                "The configuration for the {0} chain does not include a "
                "registrar.  Please set this value to the address of the "
                "deployed registrar contract.".format(self.chain_name)
            )
        return get_compiled_registrar_contract(
            self.web3,
            address=self.chain_config['registrar'],
        )

    def get_geth_process_instance(self):
        raise NotImplementedError("Must be implemented by subclasses")

    def __enter__(self, *args, **kwargs):
        self.stack.enter_context(self.geth)

        if self.geth.is_mining:
            self.geth.wait_for_dag(600)
        if self.geth.ipc_enabled:
            self.geth.wait_for_ipc(60)
        if self.geth.rpc_enabled:
            self.geth.wait_for_rpc(60)

        return self

    def __exit__(self, *exc_info):
        self.stack.close()


class LocalGethChain(BaseGethChain):
    def get_geth_process_instance(self):
        return LoggedDevGethProcess(
            project_dir=self.project.project_dir,
            blockchains_dir=self.project.blockchains_dir,
            chain_name=self.chain_name,
            overrides=self.geth_kwargs,
        )


class TemporaryGethChain(BaseGethChain):
    def get_geth_process_instance(self):
        tmp_project_dir = self.stack.enter_context(tempdir())
        blockchains_dir = get_blockchains_dir(tmp_project_dir)

        return LoggedDevGethProcess(
            project_dir=self.project.project_dir,
            blockchains_dir=blockchains_dir,
            chain_name=self.chain_name,
            overrides=self.geth_kwargs,
        )

    has_registrar = True

    @cached_property
    def registrar(self):
        RegistrarFactory = get_compiled_registrar_contract(self.web3)
        deploy_txn_hash = RegistrarFactory.deploy()
        registrar_address = get_contract_address_from_txn(
            self.web3,
            deploy_txn_hash,
            60,
        )
        registrar = RegistrarFactory(address=registrar_address)

        return registrar


class MordenChain(BaseGethChain):
    def get_geth_process_instance(self):
        return LoggedMordenGethProccess(
            project_dir=self.project.project_dir,
            geth_kwargs=self.geth_kwargs,
        )


class MainnetChain(BaseGethChain):
    def get_geth_process_instance(self):
        return LoggedMainnetGethProcess(
            project_dir=self.project.project_dir,
            geth_kwargs=self.geth_kwargs,
        )



import sys

import click

from populus.utils.filesystem import (
    is_same_path,
)
from populus.project import (
    Project,
)


CONTEXT_SETTINGS = dict(
    # Support -h as a shortcut for --help
    help_option_names=['-h', '--help'],
)


@click.group(context_settings=CONTEXT_SETTINGS)
@click.option(
    '--config',
    '-c',
    help=(
        "Specify a populus configuration file to be used.  No other "
        "configuration files will be loaded"
    ),
    type=click.File(),
    multiple=True,
)
@click.pass_context
def main(ctx, config):
    """
    Populus
    """
    project = Project(config)

    if not any(is_same_path(p, project.project_dir) for p in sys.path):
        # ensure that the project directory is in the sys.path
        sys.path.insert(0, project.project_dir)

    ctx.obj = {}
    ctx.obj['PROJECT'] = project



import os
import copy
import itertools
import operator

try:
    import configparser
except ImportError:
    from backports import configparser

from .chains import (
    get_default_ipc_path,
)


CONFIG_SEARCH_PATHS = [
    os.path.expanduser('~'),
]

PRIMARY_CONFIG_FILENAME = 'populus.ini'
CONFIG_FILENAMES = [
    PRIMARY_CONFIG_FILENAME,
]


TESTRPC_DEFAULTS = {
    'provider': 'web3.providers.rpc.TestRPCProvider',
}

MAINNET_DEFAULTS = {
    'provider': 'web3.providers.ipc.IPCProvider',
    'ipc_path': get_default_ipc_path(testnet=False),
}

MORDEN_DEFAULTS = {
    'provider': 'web3.providers.ipc.IPCProvider',
    'ipc_path': get_default_ipc_path(testnet=True),
}


OPTION_TYPES = {
    'is_external': 'getboolean',
}


class Config(configparser.ConfigParser):
    @property
    def chains(self):
        defaults = {
            'mainnet': copy.deepcopy(MAINNET_DEFAULTS),
            'morden': copy.deepcopy(MORDEN_DEFAULTS),
            'testrpc': copy.deepcopy(TESTRPC_DEFAULTS),
            'temp': {},
        }
        declared_chains = {
            section.partition(':')[2]: {
                section_key: operator.methodcaller(
                    OPTION_TYPES.get(section_key, 'get'),
                    section,
                    section_key,
                )(self)
                for section_key in self.options(section)
            }
            for section in self.sections()
            if section.startswith('chain:')
        }
        defaults.update(declared_chains)
        return defaults


def get_config_paths(project_dir, extra_paths=CONFIG_SEARCH_PATHS):
    search_paths = [project_dir] + extra_paths
    all_config_file_paths = [
        os.path.join(path, filename)
        for path, filename
        in itertools.product(search_paths, CONFIG_FILENAMES)
    ]
    return all_config_file_paths


def load_config(config_file_paths):
    config = Config()
    config.read(reversed(config_file_paths))

    return config



import os
import sys
import shutil
import fnmatch
import tempfile
import contextlib
import functools


if sys.version_info.major == 2:
    FileNotFoundError = OSError


def ensure_path_exists(dir_path):
    """
    Make sure that a path exists
    """
    if not os.path.exists(dir_path):
        os.makedirs(dir_path)
        return True
    return False


def ensure_file_exists(file_path):
    """
    Make sure that a path exists
    """
    if os.path.exists(file_path):
        return False
    base_dir = os.path.dirname(file_path)
    ensure_path_exists(base_dir)
    with open(file_path, 'w'):
        pass
    return True


def remove_file_if_exists(path):
    if os.path.isfile(path):
        os.remove(path)
        return True
    return False


def remove_dir_if_exists(path):
    if os.path.isdir(path):
        shutil.rmtree(path)
        return True
    return False


CONTRACTS_DIR = "./contracts/"


def get_contracts_dir(project_dir):
    contracts_dir = os.path.join(project_dir, CONTRACTS_DIR)
    return os.path.abspath(contracts_dir)


BUILD_DIR = "./build/"


def get_build_dir(project_dir):
    build_dir = os.path.join(project_dir, BUILD_DIR)
    ensure_path_exists(build_dir)
    return build_dir


COMPILED_CONTRACTS_FILENAME = "contracts.json"


def get_compiled_contracts_file_path(project_dir):
    build_dir = get_build_dir(project_dir)
    return os.path.join(build_dir, COMPILED_CONTRACTS_FILENAME)


BLOCKCHAIN_DIR = "./chains/"


def get_blockchains_dir(project_dir):
    blockchains_dir = os.path.abspath(os.path.join(project_dir, BLOCKCHAIN_DIR))
    ensure_path_exists(blockchains_dir)
    return blockchains_dir


MIGRATIONS_DIR = "./migrations/"


def get_migrations_dir(project_dir, lazy_create=True):
    migrations_dir = os.path.abspath(os.path.join(project_dir, MIGRATIONS_DIR))
    if lazy_create:
        init_file_path = os.path.join(migrations_dir, '__init__.py')
        ensure_path_exists(migrations_dir)
        ensure_file_exists(init_file_path)
    return migrations_dir


def is_executable_available(program):
    def is_exe(fpath):
        return os.path.isfile(fpath) and os.access(fpath, os.X_OK)

    fpath = os.path.dirname(program)
    if fpath:
        if is_exe(program):
            return True
    else:
        for path in os.environ["PATH"].split(os.pathsep):
            path = path.strip('"')
            exe_file = os.path.join(path, program)
            if is_exe(exe_file):
                return True

    return False


def recursive_find_files(base_dir, pattern):
    for dirpath, _, filenames in os.walk(base_dir):
        for filename in filenames:
            if fnmatch.fnmatch(filename, pattern):
                yield os.path.join(dirpath, filename)


@contextlib.contextmanager
def tempdir():
    directory = tempfile.mkdtemp()

    try:
        yield directory
    finally:
        shutil.rmtree(directory)


def is_same_path(p1, p2):
    n_p1 = os.path.abspath(os.path.expanduser(p1))
    n_p2 = os.path.abspath(os.path.expanduser(p2))

    try:
        return os.path.samefile(n_p1, n_p2)
    except FileNotFoundError:
        return n_p1 == n_p2


def relpath(fn):
    @functools.wraps(fn)
    def wrapper(*args, **kwargs):
        path = fn(*args, **kwargs)
        return os.path.relpath(path)
    return wrapper



import types

from solc import compile_source

from web3.utils.string import (
    force_text,
)

from populus.utils.transactions import (
    wait_for_transaction_receipt,
    get_contract_address_from_txn,
)
from populus.utils.contracts import (
    get_contract_library_dependencies,
)
from populus.utils.deploy import (
    deploy_contract,
)

from .registrar import (
    REGISTRAR_SOURCE,
)
from .deferred import (
    Address,
    Resolver,
)


class Operation(object):
    """
    Base class that all migration operations inherit from.
    """
    def execute(self, **kwargs):
        raise NotImplementedError(
            "The `execute` method must be implemented by each Operation subclass"
        )

    def deconstruct(self):
        deferred_kwargs = {
            key: getattr(self, key)
            for key in dir(self)
            if all((
                # magic methods
                not key.startswith('__'),
                # functions
                not isinstance(getattr(self, key), types.FunctionType),
                # methods
                not isinstance(getattr(self, key), types.MethodType),
                # attributes that are either not present on the parent class or
                # don't match the version found on the parent class.
                (
                    not hasattr(type(self), key) or
                    getattr(self, key) != getattr(type(self), key)
                ),
            ))
        }
        return (
            '.'.join((self.__class__.__module__, self.__class__.__name__)),
            tuple(),
            deferred_kwargs,
        )


class RunPython(Operation):
    """
    A migration operation that runs custom python code for executing operations
    that don't fit within the provided operation canvas.
    """
    def __init__(self, callback):
        self.callback = callback

    def execute(self, **kwargs):
        return self.callback(**kwargs)


class SendTransaction(Operation):
    """
    A migration operation that sends a transaction.
    """
    transaction = None
    timeout = 180

    def __init__(self, transaction, timeout=180):
        self.transaction = transaction
        self.timeout = timeout

    def execute(self, chain, **kwargs):
        resolver = Resolver(chain)

        transaction = {
            resolver(key): resolver(value)
            for key, value in resolver(self.transaction).items()
        }
        timeout = resolver(self.timeout)

        transaction_hash = chain.web3.eth.sendTransaction(transaction)
        if timeout is not None:
            wait_for_transaction_receipt(
                chain.web3, transaction_hash, timeout=timeout,
            )
        return {
            'transaction-hash': transaction_hash,
        }


class DeployContract(Operation):
    contract_name = None
    transaction = None
    timeout = 180
    libraries = None
    verify = True

    def __init__(self,
                 contract_name,
                 transaction=None,
                 arguments=None,
                 verify=True,
                 libraries=None,
                 timeout=180):
        if libraries is None:
            libraries = {}

        self.contract_name = contract_name
        self.libraries = libraries

        if timeout is None and verify:
            raise ValueError(
                "Invalid configuration.  When verifying a contracts deployment, "
                "the timeout value must be set."
            )

        if transaction is None:
            transaction = {}

        if 'data' in transaction or 'to' in transaction:
            raise ValueError(
                "Invalid configuration.  You cannot specify `data` or `to` "
                "values in `DeployContract` transactions."
            )

        if arguments is None:
            arguments = []

        self.transaction = transaction
        self.arguments = arguments
        self.verify = verify

        if timeout is not None:
            self.timeout = timeout

    def execute(self, chain, compiled_contracts, **kwargs):
        resolver = Resolver(chain)

        contract_name = resolver(self.contract_name)
        libraries = {
            resolver(key): resolver(value)
            for key, value in resolver(self.libraries).items()
        }
        transaction = {
            resolver(key): resolver(value)
            for key, value in resolver(self.transaction).items()
        }
        arguments = [
            resolver(arg) for arg in self.arguments
        ]
        timeout = resolver(self.timeout)
        verify = resolver(self.verify)

        contract_data = compiled_contracts[contract_name]
        BaseContractFactory = chain.web3.eth.contract(
            abi=contract_data['abi'],
            code=contract_data['code'],
            code_runtime=contract_data['code_runtime'],
            source=contract_data.get('source'),
        )

        all_known_contract_names = set(libraries.keys()).union(
            set(compiled_contracts.keys())
        )
        library_dependencies = get_contract_library_dependencies(
            BaseContractFactory.code,
            all_known_contract_names,
        )

        registrar = chain.registrar

        def resolve_library_link(library_name):
            registrar_key = "contract/{0}".format(library_name)

            if library_name in libraries:
                return libraries[library_name]
            elif registrar.call().exists(registrar_key):
                library_address = registrar.call().getAddress(registrar_key)
                # TODO: implement validation that this contract address is
                # in fact the library we want to link against.
                return library_address
            else:
                raise ValueError(
                    "Unable to find address for library '{0}'".format(library_name)
                )

        link_dependencies = {
            dependency_name: resolve_library_link(dependency_name)
            for dependency_name
            in library_dependencies
        }

        deploy_transaction_hash, contract_factory = deploy_contract(
            chain=chain,
            contract_name=contract_name,
            contract_factory=BaseContractFactory,
            deploy_transaction=transaction,
            deploy_arguments=arguments,
            link_dependencies=link_dependencies,
        )

        if timeout is not None:
            contract_address = get_contract_address_from_txn(
                chain.web3, deploy_transaction_hash, timeout=timeout,
            )
            if verify:
                code = force_text(chain.web3.eth.getCode(contract_address))
                expected_code = force_text(contract_factory.code_runtime)
                if code != expected_code:
                    raise ValueError(
                        "Bytecode @ {0} does not match expected contract "
                        "bytecode.\n\n"
                        "expected : '{1}'\n"
                        "actual   : '{2}'\n".format(
                            contract_address,
                            expected_code,
                            code,
                        ),
                    )
            return {
                'contract-address': contract_address,
                'deploy-transaction-hash': deploy_transaction_hash,
                'canonical-contract-address': Address.defer(
                    key='/'.join(('contract', contract_name)),
                    value=contract_address,
                ),
            }

        return {
            'deploy-transaction-hash': deploy_transaction_hash,
        }


class TransactContract(Operation):
    contract_address = None
    contract_name = None
    method_name = None
    arguments = None
    transaction = None

    timeout = 180

    def __init__(self,
                 contract_address,
                 contract_name,
                 method_name,
                 arguments=None,
                 transaction=None,
                 timeout=180):
        self.contract_address = contract_address
        self.contract_name = contract_name
        self.method_name = method_name

        if arguments is None:
            arguments = []
        self.arguments = arguments

        if transaction is None:
            transaction = {}

        self.transaction = transaction

        if timeout is not None:
            self.timeout = timeout

    def execute(self, chain, compiled_contracts, **kwargs):
        resolver = Resolver(chain)

        contract_address = resolver(self.contract_address)
        contract_name = resolver(self.contract_name)
        transaction = {
            resolver(key): resolver(value)
            for key, value in resolver(self.transaction).items()
        }
        arguments = [
            resolver(arg) for arg in self.arguments
        ]
        method_name = resolver(self.method_name)
        timeout = resolver(self.timeout)

        if not contract_address:
            raise ValueError("cannot transact without an address")

        contract_data = compiled_contracts[contract_name]
        contract = chain.web3.eth.contract(
            address=contract_address,
            abi=contract_data['abi'],
            code=contract_data['code'],
            code_runtime=contract_data['code_runtime'],
            source=contract_data.get('source'),
        )

        transactor = contract.transact(transaction)
        method = getattr(transactor, method_name)
        transaction_hash = method(*arguments)

        if timeout is not None:
            wait_for_transaction_receipt(
                chain.web3, transaction_hash, timeout=timeout,
            )

        return {
            'transaction-hash': transaction_hash,
        }


class DeployRegistrar(DeployContract):
    def __init__(self, **kwargs):
        super(DeployRegistrar, self).__init__(
            contract_name="Registrar",
            **kwargs
        )

    def execute(self, chain, **kwargs):
        kwargs.pop('compiled_contracts', None)
        compiled_contracts = compile_source(REGISTRAR_SOURCE)
        return super(DeployRegistrar, self).execute(
            chain=chain,
            compiled_contracts=compiled_contracts,
            **kwargs
        )



import os

import click

from populus.utils.filesystem import (
    ensure_path_exists
)

from .main import main


TEST_FILE_CONTENTS = """from populus.utils.transactions import (
    wait_for_transaction_receipt,
)


def test_greeter(web3, chain):
    greeter = chain.get_contract('Greeter')

    greeting = greeter.call().greet()
    assert greeting == 'Hello'


def test_custom_greeting(web3, chain):
    greeter = chain.get_contract('Greeter')

    set_txn_hash = greeter.transact().setGreeting('Guten Tag')
    wait_for_transaction_receipt(web3, set_txn_hash)

    greeting = greeter.call().greet()
    assert greeting == 'Guten Tag'
"""


GREETER_FILE_CONTENTS = """contract Greeter {
    string public greeting;

    function Greeter() {
        greeting = 'Hello';
    }

    function setGreeting(string _greeting) public {
        greeting = _greeting;
    }

    function greet() constant returns (string) {
        return greeting;
    }
}
"""


@main.command()
@click.pass_context
def init(ctx):
    """
    Generate project layout with an example contract.
    """
    project = ctx.obj['PROJECT']

    if not os.path.exists(project.primary_config_file_path):
        with open(project.primary_config_file_path, 'w') as config_file:
            config_file.write("[populus]\n")
            config_file.write("project_dir = {0}".format(
                os.path.relpath(project.project_dir)
            ))

    if ensure_path_exists(project.contracts_dir):
        click.echo(
            "Created Directory: ./{0}".format(
                os.path.relpath(project.contracts_dir)
            )
        )

    example_contract_path = os.path.join(project.contracts_dir, 'Greeter.sol')
    if not os.path.exists(example_contract_path):
        with open(example_contract_path, 'w') as example_contract_file:
            example_contract_file.write(GREETER_FILE_CONTENTS)
        click.echo("Created Example Contract: ./{0}".format(
            os.path.relpath(example_contract_path)
        ))

    tests_dir = os.path.join(project.project_dir, 'tests')
    if ensure_path_exists(tests_dir):
        click.echo("Created Directory: ./{0}".format(os.path.relpath(tests_dir)))

    example_tests_path = os.path.join(tests_dir, 'test_greeter.py')
    if not os.path.exists(example_tests_path):
        with open(example_tests_path, 'w') as example_tests_file:
            example_tests_file.write(TEST_FILE_CONTENTS)
        click.echo("Created Example Tests: ./{0}".format(
            os.path.relpath(example_tests_path)
        ))



"""
From: https://gist.github.com/fpom/92a690a8cf89cebd7d4a
"""
import os
import collections
import gevent
from gevent import pool
from gevent import queue


event = collections.namedtuple("event", ["name", "path", "isdir"])


class DirWatcher (object):
    def __init__(self, root):
        self.root = os.path.abspath(root)
        self.pool = pool.Pool()
        self.q = queue.Queue()
        self.w = {}
        self.add(root, "crawl")

    def get(self):
        return self.q.get()

    def add(self, path, evt="create"):
        if os.path.isdir(path):
            for name in os.listdir(path):
                self.add(os.path.join(path, name), evt)
        self.pool.spawn(self.watch, path)
        self.q.put(event(evt, path, os.path.isdir(path)))

    def watch(self, path):
        hub = gevent.get_hub()
        watcher = hub.loop.stat(path, 1)
        self.w[path] = watcher
        isdir = os.path.isdir(path)
        if isdir:
            old = set(os.listdir(path))
        while path in self.w:
            try:
                with gevent.Timeout(2):
                    hub.wait(watcher)
            except:
                continue
            if os.path.isdir(path):
                new = set(os.listdir(path))
                for name in new - old:
                    self.add(os.path.join(path, name))
                old = new
            elif os.path.exists(path):
                self.q.put(event("update", path, isdir))
            else:
                break
        if isdir:
            for name in old:
                self.w.pop(os.path.join(path, name), None)
        self.w.pop(path, None)
        self.q.put(event("delete", path, isdir))



import itertools
import os
import json
import re
import functools

import toposort

from web3.utils.formatting import (
    remove_0x_prefix,
)
from web3.utils.string import (
    coerce_args_to_text,
)

from populus.utils.functional import (
    compose,
)
from .filesystem import (
    get_compiled_contracts_file_path,
)


def package_contracts(contract_classes):
    _dict = {
        '__len__': lambda s: len(contract_classes),
        '__iter__': lambda s: iter(contract_classes.items()),
        '__contains__': lambda s, k: contract_classes.__contains__(k),
        '__getitem__': lambda s, k: contract_classes.__getitem__(k),
        'keys': lambda s: contract_classes.keys(),
        'values': lambda s: contract_classes.values(),
    }
    _dict.update(contract_classes)

    return type('contracts', (object,), _dict)()


def construct_contract_factories(web3, contracts):
    constructor_kwargs = {
        contract_name: {
            'code': contract_data.get('code'),
            'code_runtime': contract_data.get('code_runtime'),
            'abi': contract_data.get('abi'),
            'source': contract_data.get('source'),
            'address': contract_data.get('address'),
        } for contract_name, contract_data in contracts.items()
    }
    contract_classes = {
        name: web3.eth.contract(**contract_data)
        for name, contract_data in constructor_kwargs.items()
    }
    return package_contracts(contract_classes)


def load_compiled_contract_json(project_dir):
    compiled_contracts_path = get_compiled_contracts_file_path(project_dir)

    if not os.path.exists(compiled_contracts_path):
        raise ValueError("No compiled contracts found")

    with open(compiled_contracts_path) as contracts_file:
        contracts = json.loads(contracts_file.read())

    return contracts


DEPENDENCY_RE = re.compile((
    '__'  # Prefixed by double underscore
    '(?P<name>[a-zA-Z_](?:[a-zA-Z0-9_]{0,34}[a-zA-Z0-9])?)'  # capture the name of the dependency
    '_{0,35}'
    '__'  # End with a double underscore
))


@coerce_args_to_text
def find_link_references(bytecode):
    """
    Given bytecode, this will return all of the unlinked references from within
    the bytecode.

    The returned names may be truncated to 36 characters.
    """
    return set(DEPENDENCY_RE.findall(bytecode))


def make_link_regex(contract_name):
    """
    Returns a regex that will match embedded link references within a
    contract's bytecode.
    """
    return re.compile(
        contract_name[:36].ljust(38, "_").rjust(40, "_")
    )


def expand_shortened_reference_name(name, full_names):
    """
    If a contract dependency has a name longer than 36 characters then the name
    is truncated in the compiled but unlinked bytecode.  This maps a name to
    it's full name.
    """
    if name in full_names:
        return name

    candidates = [
        n for n in full_names if n.startswith(name)
    ]
    if len(candidates) == 1:
        return candidates[0]
    elif len(candidates) > 1:
        raise ValueError(
            "Multiple candidates found trying to expand '{0}'.  Found '{1}'. "
            "Searched '{2}'".format(
                name,
                ','.join(candidates),
                ','.join(full_names),
            )
        )
    else:
        raise ValueError(
            "Unable to expand '{0}'. "
            "Searched '{1}'".format(
                name,
                ','.join(full_names),
            )
        )


def link_bytecode(bytecode, **dependencies):
    """
    Given the bytecode for a contract, and it's dependencies in the form of
    {contract_name: address} this functino returns the bytecode with all of the
    link references replaced with the dependency addresses.
    """
    linker_fn = compose(*(
        functools.partial(
            make_link_regex(name).sub,
            remove_0x_prefix(address),
        )
        for name, address in dependencies.items()
    ))
    linked_bytecode = linker_fn(bytecode)
    return linked_bytecode


def get_contract_library_dependencies(bytecode, full_contract_names):
    """
    Given a contract bytecode and an iterable of all of the known full names of
    contracts, returns a set of the contract names that this contract bytecode
    depends on.

    To get the full dependency graph use the `get_recursive_contract_dependencies`
    function.
    """
    expand_fn = functools.partial(
        expand_shortened_reference_name,
        full_names=full_contract_names,
    )
    return {
        expand_fn(name) for name in find_link_references(bytecode)
    }


def get_shallow_dependency_graph(contracts):
    """
    Given a dictionary of compiled contract data, this returns a *shallow*
    dependency graph of each contracts explicit link dependencies.
    """
    dependencies = {
        contract_name: get_contract_library_dependencies(
            contract_data['code'],
            contracts.keys(),
        )
        for contract_name, contract_data
        in contracts.items()
        if contract_data.get('code') is not None
    }
    return dependencies


def get_contract_deploy_order(dependency_graph):
    return toposort.toposort_flatten(dependency_graph)


def get_recursive_contract_dependencies(contract_name, dependency_graph):
    """
    Recursive computation of the linker dependencies for a specific contract
    within a contract dependency graph.
    """
    return set(itertools.chain(
        dependency_graph.get(contract_name, set()), *(
            get_recursive_contract_dependencies(dep, dependency_graph)
            for dep in dependency_graph.get(contract_name, set())
        )
    ))



import os
import hashlib

from web3.utils.string import (
    is_string,
)

from populus.utils.filesystem import (
    get_contracts_dir,
    get_build_dir,
    get_compiled_contracts_file_path,
    get_blockchains_dir,
    get_migrations_dir,
    relpath,
)
from populus.utils.chains import (
    get_data_dir,
    get_chaindata_dir,
    get_dapp_dir,
    get_geth_ipc_path,
    get_nodekey_path,
)
from populus.utils.config import (
    load_config,
    get_config_paths,
    PRIMARY_CONFIG_FILENAME,
)
from populus.migrations.migration import (
    sort_migrations,
)
from populus.migrations.loading import (
    find_project_migrations,
    load_project_migrations,
)
from populus.compilation import (
    find_project_contracts,
    compile_project_contracts,
)

from populus.chain import (
    TestRPCChain,
    TemporaryGethChain,
    MordenChain,
    MainnetChain,
    LocalGethChain,
    ExternalChain,
)


class Project(object):

    #: Instance of :class:`populus.utils.Config`, a subclass of ConfigParser
    config = None

    def __init__(self, config_file_paths=None):
        self.load_config(config_file_paths)

    #
    # Config
    #
    _primary_config_file_path = None
    _config_file_paths = None

    @property
    @relpath
    def primary_config_file_path(self):
        if self._primary_config_file_path is not None:
            return self._primary_config_file_path
        return os.path.join(self.project_dir, PRIMARY_CONFIG_FILENAME)

    @primary_config_file_path.setter
    def primary_config_file_path(self, value):
        self._primary_config_file_path = value

    def write_config(self, destination_path=None):
        if destination_path is None:
            destination_path = self.primary_config_file_path

        with open(destination_path, 'w') as config_file:
            self.config.write(config_file)

        return destination_path

    def load_config(self, config_file_paths=None):
        self._config_file_paths = config_file_paths

        if not config_file_paths:
            config_file_paths = get_config_paths(os.getcwd())
        else:
            self.primary_config_file_path = config_file_paths[0]

        if is_string(config_file_paths):
            config_file_paths = [config_file_paths]

        self.config = load_config(config_file_paths)

    def reload_config(self):
        self.load_config(self._config_file_paths)

    #
    # Project
    #
    @property
    @relpath
    def project_dir(self):
        if self.config.has_option('populus', 'project_dir'):
            return self.config.get('populus', 'project_dir')
        else:
            return os.getcwd()

    #
    # Contracts
    #
    @property
    @relpath
    def contracts_dir(self):
        return get_contracts_dir(self.project_dir)

    @property
    @relpath
    def build_dir(self):
        if self.config.has_option('populus', 'build_dir'):
            return self.config.get('populus', 'build_dir')
        else:
            return get_build_dir(self.project_dir)

    @property
    @relpath
    def compiled_contracts_file_path(self):
        return get_compiled_contracts_file_path(self.project_dir)

    _cached_compiled_contracts_hash = None
    _cached_compiled_contracts = None

    def get_source_file_hash(self):
        source_file_paths = find_project_contracts(self.project_dir)
        return hashlib.md5(b''.join(
            open(source_file_path, 'rb').read()
            for source_file_path
            in source_file_paths
        )).hexdigest()

    @property
    def compiled_contracts(self):
        source_hash = self.get_source_file_hash()

        if self._cached_compiled_contracts_hash != source_hash:
            self._cached_compiled_contracts_hash = source_hash
            _, self._cached_compiled_contracts = compile_project_contracts(
                project_dir=self.project_dir,
            )
        return self._cached_compiled_contracts

    #
    # Local Blockchains
    #
    def get_chain(self, chain_name, *chain_args, **chain_kwargs):
        """
        Returns a context manager that runs a chain within the context of the
        current populus project.

        Support pre-configured chain names:

        - 'testrpc': Chain backed by an ephemeral eth-testrpc chain.
        - 'temp': Chain backed by geth running a local chain in a temporary
          directory that will be automatically deleted when the chain shuts down.
        - 'mainnet': Chain backed by geth running against the public mainnet.
        - 'morden': Chain backed by geth running against the public morden
          testnet.

        Alternatively you can specify any of the pre-configured chains from the
        project's populus.ini configuration file.

        All geth backed chains are subject to up to 10 minutes of wait time
        during first boot to generate the DAG file if the chain configured to
        mine.

        * See https://github.com/ethereum/wiki/wiki/Ethash-DAG
        * These are shared across all Ethereum nodes and live in
          ``$(HOME)/.ethash/`` folder

        To avoid this long wait time, you can manuall pre-generate the DAG with
        ``$ geth makedag 0 $HOME/.ethash``

        Example:

        .. code-block:: python

            >>> from populus.project import default_project as my_project
            >>> with my_project.get_chain('testrpc') as chain:
            ...     web3 = chain.web3
            ...     MyContract = chain.contract_factories.MyContract
            ...     # do things


        :param chain_name: The name of the chain that should be returned
        :param chain_args: Positional arguments that should be passed into the
                           chain constructor.
        :param chain_kwargs: Named arguments that should be passed into the
                             constructor

        :return: :class:`populus.chain.Chain`
        """
        if chain_name == 'testrpc':
            return TestRPCChain(self, 'testrpc', *chain_args, **chain_kwargs)
        elif chain_name == 'temp':
            return TemporaryGethChain(self, 'temp', *chain_args, **chain_kwargs)

        try:
            chain_config = self.config.chains[chain_name]
        except KeyError:
            raise KeyError(
                "Unknown chain: {0!r} - Must be one of {1!r}".format(
                    chain_name,
                    sorted(self.config.chains.keys()),
                )
            )

        combined_kwargs = dict(**chain_config)
        combined_kwargs.update(chain_kwargs)

        if chain_config.get('is_external'):
            # TODO: the chain_kwargs is really currently required to contain a
            # `web3` instance.  This isn't quite congruent with the current
            # API.
            return ExternalChain(self, chain_name, *chain_args, **combined_kwargs)

        if chain_name == 'morden':
            return MordenChain(self, 'morden', *chain_args, **combined_kwargs)
        elif chain_name == 'mainnet':
            return MainnetChain(self, 'mainnet', *chain_args, **combined_kwargs)
        else:
            return LocalGethChain(self,
                                  chain_name=chain_name,
                                  *chain_args,
                                  **combined_kwargs)

    @property
    @relpath
    def blockchains_dir(self):
        return get_blockchains_dir(self.project_dir)

    @relpath
    def get_blockchain_data_dir(self, chain_name):
        return get_data_dir(self.project_dir, chain_name)

    @relpath
    def get_blockchain_chaindata_dir(self, chain_name):
        return get_chaindata_dir(self.get_blockchain_data_dir(chain_name))

    @relpath
    def get_blockchain_dapp_dir(self, chain_name):
        return get_dapp_dir(self.get_blockchain_data_dir(chain_name))

    @relpath
    def get_blockchain_ipc_path(self, chain_name):
        return get_geth_ipc_path(self.get_blockchain_data_dir(chain_name))

    @relpath
    def get_blockchain_nodekey_path(self, chain_name):
        return get_nodekey_path(self.get_blockchain_data_dir(chain_name))

    #
    # Migrations
    #
    @property
    @relpath
    def migrations_dir(self):
        return get_migrations_dir(self.project_dir)

    @property
    def migration_files(self):
        return list((
            os.path.relpath(migration_file_path)
            for migration_file_path
            in sorted(find_project_migrations(self.project_dir))
        ))

    @property
    def migrations(self):
        return sort_migrations(
            load_project_migrations(self.project_dir),
            flatten=True,
        )



import os
import json

from populus.utils.filesystem import (
    get_contracts_dir,
    get_compiled_contracts_file_path,
    recursive_find_files,
)
from solc import (
    compile_files,
)
from solc.exceptions import (
    ContractsNotFound,
)


def find_project_contracts(project_dir):
    contracts_dir = get_contracts_dir(project_dir)

    return tuple(
        os.path.relpath(p) for p in recursive_find_files(contracts_dir, "*.sol")
    )


def write_compiled_sources(project_dir, compiled_sources):
    compiled_contract_path = get_compiled_contracts_file_path(project_dir)

    with open(compiled_contract_path, 'w') as outfile:
        outfile.write(
            json.dumps(compiled_sources,
                       sort_keys=True,
                       indent=4,
                       separators=(',', ': '))
        )
    return compiled_contract_path


def compile_project_contracts(project_dir, **compiler_kwargs):
    compiler_kwargs.setdefault('output_values', ['bin', 'bin-runtime', 'abi'])
    contract_source_paths = find_project_contracts(project_dir)
    try:
        compiled_sources = compile_files(contract_source_paths, **compiler_kwargs)
    except ContractsNotFound:
        return contract_source_paths, {}

    return contract_source_paths, compiled_sources


def compile_and_write_contracts(project_dir, **compiler_kwargs):
    contract_source_paths, compiled_sources = compile_project_contracts(
        project_dir,
        **compiler_kwargs
    )

    output_file_path = write_compiled_sources(project_dir, compiled_sources)
    return contract_source_paths, compiled_sources, output_file_path



from .main import main  # NOQA
from .chain_cmd import chain_cmd  # NOQA
from .compile_cmd import compile_contracts  # NOQA
from .deploy_cmd import deploy  # NOQA
from .init_cmd import init  # NOQA
from .config_cmd import (  # NOQA
    config,
    config_set,
    config_unset,
)
from .makemigration_cmd import make_migration  # NOQA
from .migrate_cmd import migrate  # NOQA



import click

from .main import main


def validate_key_value_pair(ctx, param, keys_and_values):
    for key_and_value in keys_and_values:
        try:
            key, value = key_and_value.split(':')
        except ValueError:
            raise click.BadParameter(
                "Values must be formatted as 'key:value'."
            )
    return keys_and_values


@main.command()
@click.pass_context
def config(ctx):
    """
    Print the current project configuration
    """
    project = ctx.obj['PROJECT']
    config = project.config

    if not any(config.options(section) for section in config.sections()):
        click.echo(
            "No configuration values set.  Use `populus config set "
            "some_key:a_value` to set configuration options."
        )

    for section in config.sections():
        if not config.options(section):
            # don't print empty sections
            continue
        click.echo("[{0}]".format(section))
        for key in config.options(section):
            click.echo("  {key} = {value}".format(
                key=key, value=config.get(section, key)
            ))


@main.command('config:set')
@click.option(
    '--section',
    '-s',
    default='populus',
)
@click.argument(
    'keys_and_values',
    nargs=-1,
    callback=validate_key_value_pair,
)
@click.pass_context
def config_set(ctx, section, keys_and_values):
    """
    Sets key/value pairs in the populus.ini configuration file. Values should
    be formatted as 'key:value'.
    """
    project = ctx.obj['PROJECT']
    config = project.config

    if not config.has_section(section):
        config.add_section(section)

    for key_and_value in keys_and_values:
        key, value = key_and_value.split(':')
        config.set(section, key, value)

    project.write_config()


@main.command('config:unset')
@click.option(
    '--section',
    '-s',
    default='populus',
)
@click.option(
    '--confirm/--no-confirm',
    is_flag=True,
    default=True,
)
@click.argument(
    'keys',
    nargs=-1,
)
@click.pass_context
def config_unset(ctx, section, confirm, keys):
    """
    Deletes the provided keys from the configuration.  To delete an entire
    section, pass in '*'
    """
    project = ctx.obj['PROJECT']
    config = project.config

    if not config.has_section(section):
        if confirm:
            confirm_msg = (
                "Are you sure you want to delete the {0!r} section?".format(
                    section
                )
            )
            if not click.prompt(confirm_msg):
                ctx.abort()
        ctx.abort("Unknown section: {0!r}".format(section))

    if len(keys) == 1 and keys[0] == "*":
        config.remove_section(section)
    else:
        for key in keys:
            config.remove_option(section, key)

    project.write_config()



import gevent

import click

from populus.utils.cli import (
    compile_project_contracts,
    watch_project_contracts,
)

from .main import main


@main.command('compile')
@click.option(
    '--watch',
    '-w',
    is_flag=True,
    help="Watch contract source files and recompile on changes",
)
@click.option(
    '--optimize',
    '-o',
    default=True,
    is_flag=True,
    help="Enable compile time optimization",
)
@click.pass_context
def compile_contracts(ctx, watch, optimize):
    """
    Compile project contracts, storing their output in `./build/contracts.json`

    Call bare to compile all contracts or specify contract names or file paths
    to restrict to only compiling those contracts.

    Pass in a file path and a contract name separated by a colon(":") to
    specify only named contracts in the specified file.
    """
    project = ctx.obj['PROJECT']

    compile_project_contracts(project, optimize=True)

    if watch:
        thread = gevent.spawn(
            watch_project_contracts,
            project=project,
            optimize=True,
        )
        thread.join()



import os
import glob
import re
from importlib import (
    import_module,
)
try:
    from importlib import invalidate_caches
except ImportError:
    from populus.utils.functional import noop as invalidate_caches

try:
    from importlib import reload
except ImportError:
    pass

from populus.utils.filesystem import (
    get_migrations_dir,
)


VALID_MIGRATION_REGEX = re.compile(
    r'^[0-9]{4}_[_a-zA-Z0-9]+\.py$',
)


def is_valid_migration_filename(filename):
    return bool(VALID_MIGRATION_REGEX.match(filename))


def find_project_migrations(project_dir):
    migrations_dir = get_migrations_dir(project_dir)
    glob_pattern = os.path.join(migrations_dir, '*.py')
    migrations = [
        os.path.relpath(migration_path, project_dir)
        for migration_path
        in glob.glob(glob_pattern)
        if is_valid_migration_filename(os.path.basename(migration_path))
    ]
    return migrations


def load_project_migrations(project_dir):
    migration_file_paths = [
        os.path.relpath(file_path, project_dir)
        for file_path in find_project_migrations(project_dir)
    ]
    migration_module_paths = [
        file_path[:-3].replace(os.sep, '.')
        for file_path in migration_file_paths
    ]

    # we need to both
    invalidate_caches()
    migration_base_paths = set([
        migration_module_path.rpartition('.')[0]
        for migration_module_path in migration_module_paths
    ])

    for migration_base_path in migration_base_paths:
        reload(import_module(migration_base_path))

    migration_modules = [
        reload(import_module(module_path))
        for module_path in migration_module_paths
    ]

    migration_classes = [
        getattr(migration_module, 'Migration')
        for migration_module in migration_modules
    ]
    return migration_classes



import click

from populus.utils.cli import (
    select_chain,
    show_chain_sync_progress,
    get_unlocked_deploy_from_address,
)
from populus.migrations.migration import (
    get_migration_classes_for_execution,
)
from populus.migrations.validation import (
    validate_migration_classes,
)

from .main import main


@main.command('migrate')
@click.argument('chain_name', nargs=1, required=False)
@click.pass_context
def migrate(ctx, chain_name):
    """
    Run project migrations
    """
    project = ctx.obj['PROJECT']

    if not project.migrations:
        raise click.Abort((
            "The project does not appear to have any migrations.  You can use "
            "the `populus makemigration command to generate project migrations"
        ))

    # Validate the project migrations
    validate_migration_classes(project.migrations)

    # Determine which chain should be used.
    if not chain_name:
        chain_name = select_chain(project)

    chain_config = project.config.chains[chain_name]

    # Determine if the chain is *migratable*
    if 'registrar' not in chain_config:
        # TODO: this should be a property of the chain object itself.
        # Something like `chain.is_ready_for_migrations`.
        if chain_name not in {'testrpc', 'temp'}:
            # ignore `testrpc` and `temp` because they lazily create their
            # registrar contracts.
            # TODO: We can present the use with the option to just initialize
            # the chain right here rather than throwing an error.
            raise click.Abort((
                "The chain {0!r} is not ready for migrations.  Please initialize this "
                "chain with the `populus migrate init` command".format(chain_name)
            ))

    with project.get_chain(chain_name) as chain:
        if chain_name in {'mainnet', 'morden'}:
            show_chain_sync_progress(chain)

        account = get_unlocked_deploy_from_address(chain)
        chain.web3.eth.defaultAccount = account

        # Wait for chain sync if this is a public network.
        if chain_name in {'mainnet', 'morden'}:
            show_chain_sync_progress(chain)

        # Determine if we have any migrations to run.
        migrations_to_execute = get_migration_classes_for_execution(
            project.migrations,
            chain,
        )

        if not migrations_to_execute:
            raise click.Abort(("All migrations have been run."))

        click.echo("Migration operations to perform:")

        for migration in migrations_to_execute:
            click.echo(''.join((
                "  ",
                migration.migration_id,
                " ({0} operations)".format(len(migration.operations)),
                ":",
            )))
            for operation_index, operation in enumerate(migration.operations):
                click.echo(''.join((
                    "    ",
                    str(operation_index),
                    " - ",
                    str(operation),
                )))

        click.echo("Executing migrations:")
        for migration in migrations_to_execute:
            click.echo(''.join((
                "  ",
                migration.migration_id,
                "... ",
            )), nl=False)
            migration.execute()
            click.echo(" DONE")



from .migration import (  # noqa
    Migration,
)
from .operations import (  # noqa
    Operation,
    RunPython,
    SendTransaction,
    DeployContract,
    TransactContract,
    DeployRegistrar,
)
from .deferred import (  # noqa
    DeferredValue,
    Address,
    Bytes32,
    String,
    UInt,
    Int,
    Bool,
)



import os

import click

from populus.migrations import (
    Migration,
)
from populus.migrations.writer import (
    get_next_migration_number,
    write_migration,
)
from populus.migrations.loading import (
    is_valid_migration_filename,
)

from .main import main


@main.command('makemigration')
@click.argument('migration_name', required=False)
@click.pass_context
def make_migration(ctx, migration_name):
    """
    Generate an empty migration.
    """
    project = ctx.obj['PROJECT']

    next_migration_number = get_next_migration_number(project.project_dir)

    if migration_name is None:
        if next_migration_number == 1:
            migration_name = 'initial'
        else:
            migration_name = 'auto'

    migration_id = "{num:04d}_{migration_name}".format(
        num=next_migration_number,
        migration_name=migration_name,
    )

    migration_filename = "{migration_id}.py".format(migration_id=migration_id)

    if not is_valid_migration_filename(migration_filename):
        ctx.fail(
            "Migration filenames may only contain letters, numbers, and "
            "underscores: {0!r}".format(migration_filename)
        )

    migration_file_path = os.path.join(project.migrations_dir, migration_filename)
    if os.path.exists(migration_file_path):
        ctx.fail((
            "Unexpectedly found duplicate migration name: {0}".format(
                os.path.abspath(migration_file_path),
            )
        ))

    compiled_contracts = project.compiled_contracts

    new_migration_dict = {
        'migration_id': migration_id,
        'operations': [],
        'dependencies': [],
        'compiled_contracts': compiled_contracts,
    }

    if project.migrations:
        new_migration_dict['dependencies'].append(
            project.migrations[-1].migration_id,
        )

    new_migration = type('Migration', (Migration,), new_migration_dict)

    with open(migration_file_path, 'w') as file_obj:
        write_migration(file_obj, new_migration)

    click.echo(
        "Wrote new migration to: {0}".format(os.path.relpath(
            migration_file_path, project.project_dir,
        ))
    )



import random
import gevent


def wait_for_transaction_receipt(web3, txn_hash, timeout=120):
    with gevent.Timeout(timeout):
        while True:
            txn_receipt = web3.eth.getTransactionReceipt(txn_hash)
            if txn_receipt is not None:
                break
            gevent.sleep(random.random())
    return txn_receipt


def wait_for_block_number(web3, block_number=1, timeout=120):
    with gevent.Timeout(timeout):
        while web3.eth.blockNumber < block_number:
            gevent.sleep(random.random())
    return web3.eth.getBlock(block_number)


def get_contract_address_from_txn(web3, txn_hash, timeout=120):
    txn_receipt = wait_for_transaction_receipt(web3, txn_hash, timeout)

    return txn_receipt['contractAddress']


def get_block_gas_limit(web3, block_identifier=None):
    if block_identifier is None:
        block_identifier = web3.eth.blockNumber
    block = web3.eth.getBlock(block_identifier)
    return block['gasLimit']


def is_account_locked(web3, account):
    try:
        web3.eth.sign(account, 'simple-test-data')
    except ValueError as err:
        return 'account is locked' in str(err)
    else:
        return False


def wait_for_unlock(web3, account=None, timeout=120):
    if account is None:
        account = web3.eth.coinbase

    with gevent.Timeout(timeout):
        while is_account_locked(web3, account):
            gevent.sleep(random.random())


def wait_for_peers(web3, peer_count=1, timeout=120):
    with gevent.Timeout(timeout):
        while web3.net.peerCount < peer_count:
            gevent.sleep(random.random())


def wait_for_syncing(web3, timeout=120):
    start_block = web3.eth.blockNumber
    with gevent.Timeout(timeout):
        while not web3.eth.syncing and web3.eth.blockNumber == start_block:
            gevent.sleep(random.random())



import sys
import random

import gevent
from gevent import socket


if sys.version_info.major == 2:
    ConnectionRefusedError = socket.error


def get_open_port():
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.bind(("", 0))
    s.listen(1)
    port = s.getsockname()[1]
    s.close()
    return port


def wait_for_connection(host, port, timeout=30):
    with gevent.Timeout(timeout):
        while True:
            s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            s.timeout = 1
            try:
                s.connect((host, port))
            except (socket.timeout, ConnectionRefusedError):
                gevent.sleep(random.random())
                continue
            else:
                break
        else:
            raise ValueError("Unable to establish HTTP connection")



import pytest

from populus.project import Project


@pytest.fixture()
def project(request):
    # This should probably be configurable using the `request` fixture but it's
    # unclear what needs to be configurable.
    return Project()


@pytest.yield_fixture()
def chain(request, project):
    # This should probably allow you to specify the test chain to be used based
    # on the `request` object.  It's unclear what the best way to do this is
    # so... punt!
    chain = project.get_chain('testrpc')

    # TODO: this should run migrations.  If `testrpc` it should be snapshotted.
    # In the future we should be able to snapshot the `geth` chains too and
    # save them for faster test runs.

    with chain:
        yield chain


@pytest.fixture()
def web3(chain):
    return chain.web3


@pytest.fixture()
def contracts(chain):
    return chain.contract_factories


@pytest.fixture()
def accounts(web3):
    return web3.eth.accounts



import os
import itertools
import random

import click

import gevent

from populus.observers import (
    DirWatcher,
)
from populus.compilation import (
    compile_and_write_contracts,
)
from .deploy import (
    deploy_contract,
)
from .transactions import (
    is_account_locked,
    get_contract_address_from_txn,
    wait_for_syncing,
    wait_for_peers,
    wait_for_unlock,
)


def select_chain(project):
    """
    Present the user with a prompt to select which of the project chains they
    want to use.
    """
    chain_options = set(project.config.chains.keys())

    choose_chain_msg = "\n".join(itertools.chain((
        "Available Chains",
        "----------------",
    ), (
        "{0} - {1}".format(chain_index, chain_name)
        for chain_index, chain_name in enumerate(sorted(chain_options))
    ), (
        "",
        "Enter ether the name, or number of the desired chain"
    )))
    chain_name = click.prompt(choose_chain_msg)
    if chain_name in chain_options:
        return chain_name
    elif chain_name.isdigit() and int(chain_name) < len(chain_options):
        return sorted(chain_options)[int(chain_name)]
    else:
        raise click.Abort(
            "Invalid choice: {0}.  Please choose from one of the "
            "provided options.".format(chain_name)
        )


def select_account(chain):
    """
    Present the user with a prompt to select which of the chain accounts they
    would like to use.
    """
    all_accounts = chain.web3.eth.accounts
    if not all_accounts:
        raise click.Abort("No accounts found on chain.")
    unlocked_accounts = {
        account for account in all_accounts
        if not is_account_locked(chain.web3, account)
    }

    pick_account_message = '\n'.join(itertools.chain((
        "Accounts",
        "-----------------",
    ), (
        "{index} - {account}{extra}".format(
            account=account,
            extra=" (unlocked)" if account in unlocked_accounts else "",
            index=index,
        ) for (index, account) in enumerate(all_accounts)
    ), (
        "",
        "Enter the account address or the number of the desired account",
    )))

    account_choice = click.prompt(
        pick_account_message,
        default=chain.web3.eth.defaultAccount,
    )

    if account_choice in set(all_accounts):
        return account_choice
    elif account_choice.isdigit() and int(account_choice) < len(all_accounts):
        return all_accounts[int(account_choice)]
    else:
        raise click.Abort(
            "Invalid choice: {0}.  Please choose from one of the "
            "provided options.".format(account_choice)
        )


def configure_chain(project, chain_name):
    """
    Interactive configuration of an existing or new chain.

    - is it external?
    - rpc or ipc?
    - rpc/ipc configuration
    - select default account (web3.eth.defaultAccount)
    """
    is_existing_chain = chain_name in project.config.chains

    chain_section_header = "chain:{chain_name}".format(chain_name=chain_name)

    if is_existing_chain:
        chain_config = dict(project.config.items(chain_section_header))
    else:
        chain_config = {}

    start_msg = "Configuring {status} chain: {chain_name}".format(
        status="existing" if is_existing_chain else "**new**",
        chain_name=chain_name,
    )
    click.echo(start_msg)
    click.echo('-' * len(start_msg))

    if is_existing_chain:
        current_configuration_msg = "\n".join(itertools.chain((
            "Current Configuration",
        ), (
            "  {key} = {value}".format(key=key, value=value)
            for key, value in chain_config.items()
        )))
        click.echo(current_configuration_msg)

    # Internal or External
    internal_or_external_msg = (
        "\n\nPopulus can run the blockchain client for you, including "
        "connecting to the public main and test networks.\n\n "
        "Should populus manage running this chain?"
    )
    is_internal = click.confirm(internal_or_external_msg, default=True)

    if not is_internal:
        chain_config['is_external'] = 'True'

    # Web3 Provider
    web3_provider_msg = (
        "\n\nWeb3 Provider Choices:\n"
        "1) IPC socket (default)\n"
        "2) RPC via HTTP\n\n"
        "How should populus connect web3.py to this chain?"
    )
    provider = click.prompt(web3_provider_msg, default='ipc')

    if provider.lower() in {'ipc', '1'}:
        chain_config['provider'] = 'web3.providers.ipc.IPCProvider'
    elif provider.lower() in {'rpc', '2'}:
        chain_config['provider'] = 'web3.providers.rpc.RPCProvider'
    else:
        raise click.Abort("Invalid response.  Allowed responses are 1/2/ipc/rpc")

    if chain_config['provider'] == 'web3.providers.ipc.IPCProvider':
        custom_ipc_path_msg = (
            "\n\nWill this blockchain be running with a non-standard `geth.ipc`"
            "path?\n\n"
        )
        if click.confirm(custom_ipc_path_msg, default=False):
            ipc_path_msg = "Path to `geth.ipc` socket?"
            ipc_path = click.prompt(ipc_path_msg)
            chain_config['ipc_path'] = ipc_path
    elif chain_config['provider'] == 'web3.providers.rpc.RPCProvider':
        custom_rpc_host = (
            "\n\nWill the RPC server be bound to `localhost`?"
        )
        if not click.confirm(custom_rpc_host, default=True):
            rpc_host_msg = "Hostname?"
            rpc_host = click.prompt(rpc_host_msg)
            chain_config['rpc_host'] = rpc_host

        custom_rpc_port = (
            "\n\nWill the RPC server be listening on port 8545?"
        )
        if not click.confirm(custom_rpc_port, default=True):
            rpc_port_msg = "Port?"
            rpc_port = click.prompt(rpc_port_msg)
            chain_config['rpc_port'] = rpc_port

    if not project.config.has_section(chain_section_header):
        project.config.add_section(chain_section_header)

    # Save config so that we can spin this chain up.
    for key, value in chain_config.items():
        project.config.set(chain_section_header, key, value)

    project.write_config()

    if project.config.chains[chain_name].get('is_external', False):
        is_chain_ready_msg = (
            "Populus needs to connect to the chain.  Press [Enter] when the "
            "chain is ready for populus"
        )
        click.confirm(is_chain_ready_msg)

    with project.get_chain(chain_name) as chain:
        web3 = chain.web3
        choose_default_account_msg = (
            "This chain will default to sending transactions from "
            "{0}.  Would you like to set a different default "
            "account?".format(web3.eth.defaultAccount)
        )
        if click.confirm(choose_default_account_msg):
            default_account = select_account(chain)
            project.config.set(
                chain_section_header, 'default_account', default_account,
            )

    click.echo(
        "Writing configuration to {0} ...".format(project.primary_config_file_path)
    )
    project.write_config()
    click.echo("Success!")


def request_account_unlock(chain, account, timeout):
    """
    Present a password prompt to unlock the given account.
    """
    if not is_account_locked(chain.web3, account):
        raise click.Abort(
            "The account `{0}` is already unlocked".format(account)
        )

    unlock_account_msg = (
        "Please provide the password to unlock account `{0}`.".format(account)
    )
    unlock_successful = chain.web3.personal.unlockAccount(
        account,
        click.prompt(unlock_account_msg, hide_input=True),
        timeout,
    )
    if not unlock_successful:
        raise click.Abort("Unable to unlock account: `{0}`".format(account))


def deploy_contract_and_verify(chain,
                               contract_name,
                               base_contract_factory=None,
                               deploy_transaction=None,
                               deploy_arguments=None,
                               link_dependencies=None):
    """
    This is a *loose* wrapper around `populus.utils.deploy.deploy_contract`
    that handles the various concerns and logging that need to be present when
    doing this as a CLI interaction.

    Deploy a contract, displaying information about the deploy process as it
    happens.  This also verifies that the deployed contract's bytecode matches
    the expected value.
    """
    if link_dependencies is None:
        link_dependencies = {}

    web3 = chain.web3

    if base_contract_factory is None:
        base_contract_factory = chain.contract_factories[contract_name]

    if is_account_locked(web3, web3.eth.defaultAccount):
        deploy_from = select_account(chain)
        if is_account_locked(web3, deploy_from):
            request_account_unlock(chain, deploy_from, None)

    # TODO: this needs to do contract linking.
    click.echo("Deploying {0}".format(contract_name))

    deploy_txn_hash, contract_factory = deploy_contract(
        chain=chain,
        contract_name=contract_name,
        contract_factory=base_contract_factory,
        deploy_transaction=deploy_transaction,
        deploy_arguments=deploy_arguments,
        link_dependencies=link_dependencies,
    )
    deploy_txn = web3.eth.getTransaction(deploy_txn_hash)

    click.echo("Deploy Transaction Sent: {0}".format(deploy_txn_hash))
    click.echo("Waiting for confirmation...")

    contract_address = get_contract_address_from_txn(
        web3=web3,
        txn_hash=deploy_txn_hash,
        timeout=180,
    )
    deploy_receipt = web3.eth.getTransactionReceipt(deploy_txn_hash)

    click.echo((
        "\n"
        "Transaction Mined\n"
        "=================\n"
        "Tx Hash      : {0}\n"
        "Address      : {1}\n"
        "Gas Provided : {2}\n"
        "Gas Used     : {3}\n\n".format(
            deploy_txn_hash,
            contract_address,
            deploy_txn['gas'],
            deploy_receipt['gasUsed'],
        )
    ))

    # Verification
    deployed_code = web3.eth.getCode(contract_address)

    if contract_factory.code_runtime:
        click.echo("Verifying deployed bytecode...")
        is_bytecode_match = deployed_code == contract_factory.code_runtime
        if is_bytecode_match:
            click.echo(
                "Verified contract bytecode @ {0} matches expected runtime "
                "bytecode".format(contract_address)
            )
        else:
            click.echo(
                "Bytecode @ {0} does not match expected contract bytecode.\n\n"
                "expected : '{1}'\n"
                "actual   : '{2}'\n".format(
                    contract_address,
                    contract_factory.code_runtime,
                    deployed_code,
                ),
                err=True,
            )
            raise click.Abort("Error deploying contract")
    else:
        click.echo(
            "No runtime available.  Falling back to verifying non-empty "
            "bytecode."
        )
        if len(deployed_code) <= 2:
            click.echo(
                "Bytecode @ {0} is unexpectedly empty.".format(contract_address),
                err=True,
            )
            raise click.Abort("Error deploying contract")
        else:
            click.echo(
                "Verified bytecode @ {0} is non-empty".format(contract_address)
            )
    return contract_factory(address=contract_address)


def show_chain_sync_progress(chain):
    """
    Display the syncing status of a chain as a progress bar
    """
    web3 = chain.web3

    if not web3.net.peerCount:
        click.echo("Waiting for peer connections.")
        try:
            wait_for_peers(web3, timeout=240)
        except gevent.Timeout:
            raise click.Abort("Never connected to any peers.")

    if not web3.eth.syncing:
        click.echo("Waiting for synchronization to start.")
        try:
            wait_for_syncing(web3, timeout=240)
        except gevent.Timeout:
            raise click.Abort("Chain synchronization never started.")

    starting_block = web3.eth.syncing['startingBlock']

    while True:
        sync_data = web3.eth.syncing

        if not sync_data:
            break

        highest_block = sync_data['highestBlock']
        blocks_to_sync = highest_block - starting_block

        with click.progressbar(length=blocks_to_sync, label="Syncing") as bar:

            while highest_block == sync_data['highestBlock']:
                sync_data = web3.eth.syncing

                if not sync_data:
                    break

                current_block = sync_data['currentBlock']
                blocks_to_sync = highest_block - starting_block

                position = current_block - starting_block

                if position:
                    bar.update(position)

                if current_block >= highest_block:
                    break

                gevent.sleep(random.random())
            else:
                # start a new progress bar with the new `highestBlock`
                continue

            # break out of the outer loop
            break


def get_unlocked_deploy_from_address(chain):
    """
    Combination of other utils to get the address deployments should come from.

    Defaults to the one set in the config.
    If not set, asks for one.
    If not in config, asks if it should be set as default.
    If not unlocked, askes for password to unlock.
    """
    web3 = chain.web3
    chain_config = chain.chain_config
    chain_name = chain.chain_name
    chain_section_name = "chain:{name}".format(name=chain_name)
    project = chain.project
    config = project.config

    # Choose the address we should deploy from.
    if 'deploy_from' in chain_config:
        account = chain_config['deploy_from']
        if account not in web3.eth.accounts:
            raise click.Abort(
                "The chain {0!r} is configured to deploy from account {1!r} "
                "which was not found in the account list for this chain. "
                "Please ensure that this account exists.".format(
                    chain_name,
                    account,
                )
            )
    else:
        account = select_account(chain)
        set_as_deploy_from_msg = (
            "Would you like set the address '{0}' as the default"
            "`deploy_from` address for the '{1}' chain?".format(
                account,
                chain_name,
            )
        )
        if click.confirm(set_as_deploy_from_msg):
            if not config.has_section(chain_section_name):
                config.add_section(chain_section_name)
            config.set(chain_section_name, 'deploy_from', account)
            click.echo(
                "Wrote updated chain configuration to '{0}'".format(project.write_config())
            )

    # Unlock the account if needed.
    if is_account_locked(web3, account):
        try:
            # in case the chain is still spinning up, give it a moment to
            # unlock itself.
            wait_for_unlock(web3, account, 2)
        except gevent.Timeout:
            request_account_unlock(chain, account, None)

    return account


def compile_project_contracts(project, optimize=True):
    click.echo("============ Compiling ==============")
    click.echo("> Loading source files from: ./{0}\n".format(project.contracts_dir))

    result = compile_and_write_contracts(project.project_dir, optimize=optimize)
    contract_source_paths, compiled_sources, output_file_path = result

    click.echo("> Found {0} contract source files".format(
        len(contract_source_paths)
    ))
    for path in contract_source_paths:
        click.echo("- {0}".format(os.path.relpath(path)))
    click.echo("")
    click.echo("> Compiled {0} contracts".format(len(compiled_sources)))

    for contract_name in sorted(compiled_sources.keys()):
        click.echo("- {0}".format(contract_name))

    click.echo("")
    click.echo(
        "> Wrote compiled assets to: ./{0}".format(
            os.path.relpath(output_file_path)
        )
    )


def watch_project_contracts(project, **compile_kwargs):
    watcher = DirWatcher(project.contracts_dir)

    last_hash = project.get_source_file_hash()

    try:
        while True:
            event = watcher.get()
            if event.name in {'update', 'add', 'create'}:
                current_hash = project.get_source_file_hash()
                if current_hash == last_hash:
                    continue
                last_hash = current_hash
                click.echo("Change detected in: {e.path}".format(e=event))
                compile_project_contracts(project, **compile_kwargs)
    except KeyboardInterrupt:
        pass



from __future__ import unicode_literals
import itertools
import functools

from web3.utils.encoding import (
    decode_hex,
)
from web3.utils.types import (
    is_string,
)

from populus.utils.types import (
    is_hex_address,
    is_hex_transaction_hash,
)
from populus.utils.transactions import (
    wait_for_transaction_receipt,
)


class DeferredValue(object):
    chain = None

    def __init__(self, chain, **kwargs):
        self.chain = chain
        for key, value in kwargs.items():
            if not hasattr(self, key):
                raise TypeError(
                    "{0}() received an invalid keyword {1!r}. as_view only "
                    "accepts arguments that are already attributes of the "
                    "class.".format(self.__class__.__name__, key)
                )
            setattr(self, key, value)

    @classmethod
    def defer(cls, **initkwargs):
        for key in initkwargs:
            if not hasattr(cls, key):
                raise TypeError(
                    "{0}() received an invalid keyword {1!r}. as_view only "
                    "accepts arguments that are already attributes of the "
                    "class.".format(cls.__name__, key)
                )
        deferred_type = type(cls.__name__, (cls,), initkwargs)
        # without assigning the module here anytime a class is deferred, the
        # deferred class is assigned this module rather than the original
        # module that the class was defined.
        deferred_type.__module__ = cls.__module__
        return deferred_type

    @classmethod
    def deconstruct(cls):
        deferred_kwargs = {
            key: getattr(cls, key)
            for key in dir(cls)
            if (
                not key.startswith('__') and
                not callable(getattr(cls, key)) and
                getattr(cls, key) != getattr(super(cls, cls), key)
            )
        }
        return (
            '.'.join((cls.__module__, cls.__name__, 'defer')),
            tuple(),
            deferred_kwargs,
        )

    def get(self):
        raise NotImplementedError("This method must be implemented by subclasses")


def resolve_if_deferred_value(value, chain):
    if isinstance(value, DeferredValue):
        return value.get()
    elif isinstance(value, type) and issubclass(value, RegistrarValue):
        return value(chain).get()
    else:
        return value


def Resolver(chain):
    return functools.partial(resolve_if_deferred_value, chain=chain)


ALLOWED_VALUE_TYPES = {
    'string'
    'bytes32',
    'address',
    'uint256',
    'int256',
    'bool',
}


class RegistrarValue(DeferredValue):
    key = None
    value_type = None
    value = None

    def exists(self, key):
        return self.chain.registrar.call().exists(key)

    def get(self):
        if not self.exists(self.key):
            raise KeyError(
                "The given key is not set on the registrar: `{0}`".format(self.key)
            )

        caller = self.chain.registrar.call()

        if self.value_type == 'string':
            return caller.getString(self.key)
        elif self.value_type == 'bytes32':
            return caller.get(self.key)
        elif self.value_type == 'address':
            return caller.getAddress(self.key)
        elif self.value_type == 'uint256':
            return caller.getUInt(self.key)
        elif self.value_type == 'int256':
            return caller.getInt(self.key)
        elif self.value_type == 'bool':
            return caller.getBool(self.key)

        raise ValueError("`value_type` must be one of {0}.  Got: {1}".format(
            ', '.join(sorted(ALLOWED_VALUE_TYPES)),
            self.value_type,
        ))

    def set(self, value=None, timeout=120):
        if value is None:
            if self.value is not None:
                value = self.value
            else:
                raise ValueError("No value provided")

        if not is_string(self.key):
            raise ValueError("Invalid key type.  Expected string, got: {0!r}".format(
                type(self.key)
            ))

        transactor = self.chain.registrar.transact()

        if self.value_type == 'string':
            set_txn_hash = transactor.setString(self.key, value)
        elif self.value_type == 'bytes32':
            set_txn_hash = transactor.set(self.key, value)
        elif self.value_type == 'address':
            set_txn_hash = transactor.setAddress(self.key, value)
        elif self.value_type == 'uint256':
            set_txn_hash = transactor.setUInt(self.key, value)
        elif self.value_type == 'int256':
            set_txn_hash = transactor.setInt(self.key, value)
        elif self.value_type == 'bool':
            set_txn_hash = transactor.setBool(self.key, value)
        else:
            raise ValueError("`value_type` must be one of {0}.  Got: {1}".format(
                ', '.join(sorted(ALLOWED_VALUE_TYPES)),
                self.value_type,
            ))

        if timeout is not None:
            wait_for_transaction_receipt(
                web3=self.chain.web3,
                txn_hash=set_txn_hash,
                timeout=timeout,
            )

        return set_txn_hash


class Address(RegistrarValue):
    value_type = 'address'


class Bytes32(RegistrarValue):
    value_type = 'bytes32'


class UInt(RegistrarValue):
    value_type = 'uint256'


class Int(RegistrarValue):
    value_type = 'int256'


class String(RegistrarValue):
    value_type = 'string'


class Bool(RegistrarValue):
    value_type = 'bool'


def generate_registrar_value_setters(receipt, prefix=None):
    if prefix is None:
        prefix = []

    if is_string(prefix):
        prefix = [prefix]

    if isinstance(receipt, RegistrarValue):
        raise ValueError("Receipt should not be instantiated at this point")
    elif is_hex_address(receipt):
        # Special case for addresses
        return [
            Address.defer(key='/'.join(prefix), value=receipt)
        ]
    elif is_hex_transaction_hash(receipt):
        # Special case for transaction hashes and addresses.
        return [
            Bytes32.defer(key='/'.join(prefix), value=decode_hex(receipt))
        ]
    elif isinstance(receipt, type) and issubclass(receipt, RegistrarValue):
        return [
            receipt.defer(
                key=receipt.key or '/'.join(prefix),
                value_type=receipt.value_type,
                value=receipt.value,
            )
        ]
    elif isinstance(receipt, dict):
        return list(itertools.chain.from_iterable([
            generate_registrar_value_setters(value, prefix + [key])
            for key, value in receipt.items()
        ]))
    elif isinstance(receipt, (list, tuple)):
        return list(itertools.chain.from_iterable([
            generate_registrar_value_setters(value, prefix + [str(index)])
            for index, value in enumerate(receipt)
        ]))
    else:
        raise ValueError(
            "Invalid type.  Must be one of transaction hash, address, "
            "ReceiptValue, dict, or list"
        )



import random
import functools

import gevent
import click

from populus.utils.filesystem import (
    ensure_path_exists,
)
from populus.utils.cli import (
    select_chain,
    configure_chain,
    deploy_contract_and_verify,
    show_chain_sync_progress,
    get_unlocked_deploy_from_address,
)
from populus.chain import (
    BaseGethChain,
    reset_chain,
)

from .main import main


@main.group('chain')
@click.pass_context
def chain_cmd(ctx):
    """
    Manage and run ethereum blockchains.
    """
    pass


@chain_cmd.command('reset')
@click.argument('chain_name', nargs=1, default="default")
@click.option('--confirm/--no-confirm', default=True)
@click.pass_context
def chain_reset(ctx, chain_name, confirm):
    """
    Reset a chain removing all chain data and resetting it to it's genesis
    state..
    """
    project = ctx.obj['PROJECT']

    if confirm:
        confirmation_message = (
            "Are you sure you want to reset blockchain {0}: {1}".format(
                chain_name,
                project.get_blockchain_data_dir(chain_name),
            )
        )
        if not click.confirm(confirmation_message):
            raise ctx.abort()
    reset_chain(project.get_blockchain_data_dir(chain_name))


@chain_cmd.command('run')
@click.argument('chain_name', nargs=1, default="default")
@click.option('--mine/--no-mine', default=True)
@click.option(
    '--verbosity', default=5,
    help="""
    Set verbosity of the logging output. Default is 5, Range is 0-6.
    """)
@click.pass_context
def chain_run(ctx, chain_name, mine, verbosity):
    """
    Run the named chain.
    """
    project = ctx.obj['PROJECT']

    chain = project.get_chain(chain_name)

    if isinstance(chain, BaseGethChain):
        chain.geth.register_stdout_callback(click.echo)
        chain.geth.register_stderr_callback(functools.partial(click.echo, err=True))

    with chain:
        try:
            while True:
                gevent.sleep(random.random())
        except KeyboardInterrupt:
            pass


@chain_cmd.command('config')
@click.argument('chain_name', nargs=1)
@click.pass_context
def chain_configure(ctx, chain_name):
    """
    Configure a blockchain
    """
    project = ctx.obj['PROJECT']

    configure_chain(project, chain_name)


@chain_cmd.command('init')
@click.argument('chain_name', nargs=1, required=False)
@click.pass_context
def chain_init(ctx, chain_name):
    """
    Prepare the current chain for migrations.

    contract onto this chain as well as
    """
    project = ctx.obj['PROJECT']

    ensure_path_exists(project.migrations_dir)

    # Determine which chain should be used.
    if not chain_name:
        chain_name = select_chain(project)

    chain_section_name = "chain:{0}".format(chain_name)

    if chain_name == 'testrpc':
        ctx.abort("Cannot initialize the {0!r} chain".format(chain_name))

    # The `mainnet` and `morden` chains have default configurations.  If the
    # user is working on one of these chains then we need to add the section
    # header so that we can write new config to it.
    if not project.config.has_section(chain_section_name):
        project.config.add_section(chain_section_name)

    chain = project.get_chain(chain_name)

    if 'registrar' not in chain.chain_config:
        # We need to deploy the registrar
        with chain:
            web3 = chain.web3

            if chain_name in {'mainnet', 'morden'}:
                show_chain_sync_progress(chain)

            account = get_unlocked_deploy_from_address(chain)

            # Configure web3 to now send from our chosen account by default
            web3.eth.defaultAccount = account

            # Deploy the registrar
            RegistrarFactory = chain.RegistrarFactory
            registrar = deploy_contract_and_verify(chain,
                                                   contract_name='Registrar',
                                                   base_contract_factory=RegistrarFactory)

            # TODO: set the value in the registrar.

            # Write the registrar address to the chain config
            project.config.set(chain_section_name, 'registrar', registrar.address)
            config_file_path = project.write_config()

            click.echo("Wrote updated chain configuration to {0!r}".format(
                config_file_path,
            ))

    click.echo("The '{0}' blockchain is ready for migrations.".format(
        chain_name
    ))



import click

from collections import OrderedDict

from populus.utils.cli import (
    select_chain,
    show_chain_sync_progress,
    get_unlocked_deploy_from_address,
    deploy_contract_and_verify,
)
from populus.utils.deploy import (
    get_deploy_order,
)
from populus.utils.transactions import (
    wait_for_transaction_receipt,
)
from populus.migrations.registrar import (
    get_contract_from_registrar,
)

from .main import main


def echo_post_deploy_message(web3, deployed_contracts):
    # TODO: update this message.
    message = (
        "========== Deploy Completed ==========\n"
        "Deployed {n} contracts:"
    ).format(
        n=len(deployed_contracts),
    )
    click.echo(message)
    for contract_name, deployed_contract in deployed_contracts:
        deploy_receipt = web3.eth.getTransactionReceipt(deployed_contract.deploy_txn_hash)
        gas_used = deploy_receipt['gasUsed']
        deploy_txn = web3.eth.getTransaction(deploy_receipt['transactionHash'])
        gas_provided = deploy_txn['gas']
        click.echo("- {0} ({1}) gas: {2} / {3}".format(
            contract_name,
            deployed_contract.address,
            gas_used,
            gas_provided,
        ))


@main.command('deploy')
@click.option(
    'deploy_from',
    '--deploy-from',
    '-d',
    help=(
        "Specifies the account that should be used for deploys.  You can "
        "specify either the full account address, or the integer 0 based index "
        "of the account in the account list."
    ),
)
@click.option(
    'chain_name',
    '--chain',
    '-c',
    help=(
        "Specifies the chain that contracts should be deployed to. The chains "
        "mainnet' and 'morden' are pre-configured to connect to the public "
        "networks.  Other values should be predefined in your populus.ini"
    ),
)
@click.argument('contracts_to_deploy', nargs=-1)
@click.pass_context
def deploy(ctx, chain_name, deploy_from, contracts_to_deploy):
    """
    Deploys the specified contracts to a chain.
    """
    project = ctx.obj['PROJECT']

    # Determine which chain should be used.
    if not chain_name:
        chain_name = select_chain(project)

    compiled_contracts = project.compiled_contracts

    if contracts_to_deploy:
        # validate that we *know* about all of the contracts
        unknown_contracts = set(contracts_to_deploy).difference(
            compiled_contracts.keys()
        )
        if unknown_contracts:
            raise click.Abort(
                "Some contracts specified for deploy were not found in the "
                "compiled project contracts.  These contracts could not be found "
                "'{0}'.  Searched these known contracts '{1}'".format(
                    ', '.join(sorted(unknown_contracts)),
                    ', '.join(sorted(compiled_contracts.keys())),
                )
            )
    else:
        # prompt the user to select the desired contracts they want to deploy.
        # Potentially display the currently deployed status.
        raise click.Abort(
            "Not Implemented.  Please specify which contracts you wish to "
            "deploy"
        )

    chain = project.get_chain(chain_name)
    deployed_contracts = OrderedDict()

    with chain:
        web3 = chain.web3

        if chain_name in {'mainnet', 'morden'}:
            show_chain_sync_progress(chain)

        if deploy_from is None:
            deploy_from = get_unlocked_deploy_from_address(chain)
        elif deploy_from not in web3.eth.accounts:
            try:
                deploy_from = web3.eth.accounts[int(deploy_from)]
            except IndexError:
                raise click.Abort(
                    "Unknown deploy_from account: {0}".format(deploy_from)
                )

        web3.eth.defaultAccount = deploy_from

        # Get the deploy order.
        deploy_order = get_deploy_order(
            contracts_to_deploy,
            compiled_contracts,
        )

        # Display Start Message Info.
        starting_msg = (
            "Beginning contract deployment.  Deploying {0} total contracts ({1} "
            "Specified, {2} because of library dependencies)."
            "\n\n" +
            (" > ".join(deploy_order.keys()))
        ).format(
            len(deploy_order),
            len(contracts_to_deploy),
            len(deploy_order) - len(contracts_to_deploy),
        )
        click.echo(starting_msg)

        for contract_name, contract_factory in deploy_order.items():
            link_dependencies = {
                contract_name: contract.address
                for contract_name, contract
                in deployed_contracts.items()
            }

            # Check if we already have an existing deployed version of that
            # contract (via the registry).  For each of these, prompt the user
            # if they would like to use the existing version.
            if contract_name not in contracts_to_deploy and chain.has_registrar:
                # TODO: this block should be a standalone cli util.
                existing_contract = get_contract_from_registrar(
                    chain=chain,
                    contract_name=contract_name,
                    contract_factory=contract_factory,
                    link_dependencies=link_dependencies,
                )
                if existing_contract:
                    found_existing_contract_prompt = (
                        "Found existing version of {name} in registrar. "
                        "Would you like to use the previously deployed "
                        "contract @ {address}?".format(
                            name=contract_name,
                            address=existing_contract.address,
                        )
                    )
                    if click.prompt(found_existing_contract_prompt):
                        deployed_contracts[contract_name] = existing_contract
                        continue

            # We don't have an existing version of this contract available so
            # deploy it.
            contract = deploy_contract_and_verify(
                chain,
                contract_name=contract_name,
                link_dependencies=link_dependencies,
            )

            if chain.has_registrar:
                # TODO: this block should be a standalone cli util.
                contract_key = 'contract/{name}'.format(name=contract_name)
                register_txn_hash = chain.registrar.transact().setAddress(
                    contract_key, contract.address
                )
                register_msg = (
                    "Registering contract '{name}' @ {address} "
                    "in registrar in txn: {txn_hash} ...".format(
                        name=contract_name,
                        address=contract.address,
                        txn_hash=register_txn_hash,
                    )
                )
                click.echo(register_msg, nl=False)
                wait_for_transaction_receipt(web3, register_txn_hash, 180)
                click.echo(' DONE')
            deployed_contracts[contract_name] = contract

        # TODO: fix this message.
        success_msg = (
            "Deployment Successful."
        )
        click.echo(success_msg)



from __future__ import unicode_literals

import os
import functools
import codecs
from io import StringIO
from collections import OrderedDict

from web3.utils.string import force_text
from web3.utils.types import (
    is_bytes,
    is_text,
)

from populus.utils.types import (
    is_primitive_type,
)
from populus.utils.module_loading import (
    split_at_longest_importable_path,
)
from .loading import (
    find_project_migrations,
)


def get_next_migration_number(project_dir):
    migration_paths = find_project_migrations(project_dir)
    filenames = [
        os.path.basename(migration_file_path)
        for migration_file_path
        in migration_paths
    ]
    migration_numbers = [
        int(filename.partition('_')[0]) for filename in filenames
    ] + [0]
    return max(migration_numbers) + 1


def indent(text, level=0):
    return "    " * level + text


def serializer_fn(fn):
    @functools.wraps(fn)
    def inner(value, level=0, prefix='', suffix='', newline=True, *args, **kwargs):
        imports = set()
        serialized_value = StringIO()

        if prefix is not None:
            serialized_value.write(indent(prefix, level))

        inner_imports, inner_serialized_value = fn(
            value=value,
            level=level,
            *args,
            **kwargs
        )

        imports |= inner_imports
        serialized_value.write(inner_serialized_value)

        if suffix:
            serialized_value.write(suffix)

        if newline:
            serialized_value.write('\n')
        return imports, serialized_value.getvalue()
    return inner


@serializer_fn
def serialize_primitive_type(value, *args, **kwargs):
    if is_bytes(value):
        return set(), "b'{0}'".format(
            force_text(codecs.encode(force_text(value), 'unicode_escape'))
        )
    elif is_text(value):
        return set(), "'{0}'".format(
            force_text(codecs.encode(value, 'unicode_escape'))
        )
    else:
        return set(), force_text(repr(value))


@serializer_fn
def serialize_list(value, level=0, *args, **kwargs):
    if not value:
        return set(), '[]'

    imports = set()
    serialized_value = StringIO()

    serialized_value.write('[\n')

    for item in value:
        item_imports, item_serialized_value = serialize(
            item,
            level=level + 1,
            suffix=",",
            *args,
            **kwargs
        )

        imports |= item_imports
        serialized_value.write(item_serialized_value)

    serialized_value.write(indent(']', level))

    return imports, serialized_value.getvalue()


@serializer_fn
def serialize_dict(value, level=0, *args, **kwargs):
    if not value:
        return set(), '{}'

    ordered_value = OrderedDict(sorted(value.items(), key=lambda kv: kv[0]))

    imports = set()
    serialized_value = StringIO()

    serialized_value.write('{\n')

    for key, item in ordered_value.items():
        inner_kwargs = dict(**kwargs)
        inner_kwargs.pop('newline', None)

        key_imports, key_serialized_value = serialize(
            key,
            level=level + 1,
            newline=False,
            *args,
            **inner_kwargs
        )

        item_imports, item_serialized_value = serialize(
            item,
            level=level + 1,
            prefix=None,
            newline=False,
            *args,
            **inner_kwargs
        )

        imports |= key_imports
        imports |= item_imports

        serialized_value.write(key_serialized_value)
        serialized_value.write(": ")
        serialized_value.write(item_serialized_value)
        serialized_value.write(",\n")

    serialized_value.write(indent('}', level))

    return imports, serialized_value.getvalue()


@serializer_fn
def serialize_deconstructable(value, level=0, *args, **kwargs):
    import_path, construct_args, construct_kwargs = value.deconstruct()

    ordered_construct_kwargs = OrderedDict(
        sorted(construct_kwargs.items(), key=lambda kv: kv[0])
    )

    import_part, constructor_part = split_at_longest_importable_path(import_path)
    # TODO: This *effectively* just re-joins the two parts but that is ok for
    # now.  Ideally later I can figure out how to do imports in the "from a.b.c
    # import d" but this will do for now.
    constructor = '.'.join((import_part, constructor_part))

    imports = {import_part}
    serialized_value = StringIO()

    serialized_value.write(constructor)
    if not construct_args and not construct_kwargs:
        serialized_value.write("()")
        return imports, serialized_value.getvalue()

    serialized_value.write("(\n")

    inner_kwargs = dict(**kwargs)
    inner_kwargs.pop('suffix', None)

    for construct_arg in construct_args:
        arg_imports, arg_serialized_value = serialize(
            construct_arg,
            level=level + 1,
            suffix=",",
            *args,
            **inner_kwargs
        )

        imports |= arg_imports
        serialized_value.write(arg_serialized_value)

    for construct_kwarg_name, construct_kwarg_value in ordered_construct_kwargs.items():
        kwarg_imports, kwarg_serialized_value = serialize_assignment(
            construct_kwarg_name,
            construct_kwarg_value,
            level=level + 1,
            suffix=",",
            assignment_operator="=",
            *args,
            **kwargs
        )

        imports |= kwarg_imports
        serialized_value.write(kwarg_serialized_value)

    serialized_value.write(indent(")", level))
    return imports, serialized_value.getvalue()


def serialize(value, *args, **kwargs):
    if hasattr(value, 'deconstruct') and callable(value.deconstruct):
        return serialize_deconstructable(value, *args, **kwargs)
    elif is_primitive_type(value):
        return serialize_primitive_type(value, *args, **kwargs)
    elif isinstance(value, list):
        return serialize_list(value, *args, **kwargs)
    elif isinstance(value, dict):
        return serialize_dict(value, *args, **kwargs)
    else:
        raise ValueError("Unsupported type: {0!r}".format(type(value)))


def serialize_assignment(name, value, assignment_operator=" = ", *args, **kwargs):
    return serialize(
        value,
        prefix="{0}{1}".format(name, assignment_operator),
        *args,
        **kwargs
    )


MIGRATION_PROPS = (
    'migration_id',
    'dependencies',
    'operations',
    'compiled_contracts',
)


def write_migration(file_obj, migration_class):
    header = StringIO()
    body = StringIO()

    imports = set()

    body.write("class Migration(migrations.Migration):\n\n")

    for prop_name in MIGRATION_PROPS:
        prop_imports, prop_serialized_value = serialize_assignment(
            prop_name,
            getattr(migration_class, prop_name),
            level=1,
        )

        imports |= prop_imports
        body.write(prop_serialized_value)

    header.write("# -*- coding: utf-8 -*-\n")
    header.write("from __future__ import unicode_literals\n")
    header.write("\n")
    header.write("from populus import migrations\n")
    header.write("\n")

    header.writelines([
        "import {import_path}\n".format(import_path=import_path)
        for import_path in sorted(imports)
    ])

    file_obj.write(header.getvalue())
    file_obj.write("\n")
    file_obj.write(body.getvalue())



from solc import compile_source

from populus.utils.contracts import (
    link_bytecode,
)


REGISTRAR_SOURCE = """contract Registrar {
    address public owner;

    function Registrar() {
        owner = msg.sender;
    }

    modifier onlyowner { if (msg.sender != owner) throw; _ }

    function transferOwner(address newOwner) public onlyowner {
        owner = newOwner;
    }

    Registrar public parent;

    function setParent(address parentAddress) public onlyowner {
        parent = Registrar(parentAddress);
    }

    mapping (bytes32 => bytes32) records;
    mapping (bytes32 => string) stringRecords;
    mapping (bytes32 => bool) recordExists;


    function set(string key, bytes32 value) public onlyowner {
        // Compute the fixed length key
        bytes32 _key = sha3(key);

        // Set the value
        records[_key] = value;
        recordExists[_key] = true;
    }

    function get(string key) constant returns (bytes32) {
        // Compute the fixed length key
        bytes32 _key = sha3(key);

        if (!recordExists[_key]) {
            if (address(parent) == 0x0) {
                // Do return unset keys
                throw;
            } else {
                // Delegate to the parent.
                return parent.get(key);
            }
        }

        return records[_key];
    }

    function exists(string key) constant returns (bool) {
        // Compute the fixed length key
        bytes32 _key = sha3(key);

        return recordExists[_key];
    }

    function setAddress(string key, address value) public onlyowner {
        set(key, bytes32(value));
    }

    function getAddress(string key) constant returns (address) {
        return address(get(key));
    }

    function setUInt(string key, uint value) public onlyowner {
        set(key, bytes32(value));
    }

    function getUInt(string key) constant returns (uint) {
        return uint(get(key));
    }

    function setInt(string key, int value) public onlyowner {
        set(key, bytes32(value));
    }

    function getInt(string key) constant returns (int) {
        return int(get(key));
    }

    function setBool(string key, bool value) public onlyowner {
        if (value) {
            set(key, bytes32(0x1));
        } else {
            set(key, bytes32(0x0));
        }
    }

    function getBool(string key) constant returns (bool) {
        return get(key) != bytes32(0x0);
    }

    function setString(string key, string value) public onlyowner {
        bytes32 valueHash = sha3(value);
        set(key, valueHash);
        stringRecords[valueHash] = value;
    }

    function getString(string key) public returns (string) {
        bytes32 valueHash = get(key);
        return stringRecords[valueHash];
    }
}
"""


def get_compiled_registrar_contract(web3, address=None):
    compiled_contracts = compile_source(REGISTRAR_SOURCE)
    contract_data = compiled_contracts['Registrar']
    return web3.eth.contract(
        address=address,
        abi=contract_data['abi'],
        code=contract_data['code'],
        code_runtime=contract_data['code_runtime'],
        source=REGISTRAR_SOURCE,
    )


def get_contract_from_registrar(chain,
                                contract_name,
                                contract_factory,
                                link_dependencies=None):
    if link_dependencies is None:
        link_dependencies = {}

    web3 = chain.web3
    registrar = chain.registrar
    registrar_key = 'contract/{name}'.format(name=contract_name)

    if not registrar.call().exists(registrar_key):
        return None

    contract_address = registrar.call().getAddress(registrar_key)

    expected_runtime = link_bytecode(
        contract_factory.code_runtime,
        **{
            contract_name: contract.address
            for contract_name, contract
            in link_dependencies.items()
        }
    )
    actual_runtime = web3.eth.getCode(contract_address)

    # If the runtime doesn't match then don't choose it.
    if actual_runtime != expected_runtime:
        return None

    return contract_factory(address=contract_address)



import random
import signal

import gevent


def wait_for_popen(proc, max_wait=5):
    with gevent.Timeout(max_wait):
        while True:
            if proc.poll() is None:
                gevent.sleep(random.random())
            else:
                break


def kill_proc(proc):
    try:
        if proc.poll() is None:
            proc.send_signal(signal.SIGINT)
            wait_for_popen(proc, 5)
        if proc.poll() is None:
            proc.terminate()
            wait_for_popen(proc, 2)
        if proc.poll() is None:
            proc.kill()
            wait_for_popen(proc, 1)
    except KeyboardInterrupt:
        proc.kill()



import re


from web3.utils.types import (
    is_string,
    is_integer,
    is_boolean,
)
from web3.utils.string import (
    force_text,
)


def is_primitive_type(value):
    return any((
        value is None,
        is_boolean(value),
        is_string(value),
        is_integer(value),
        isinstance(value, float),
    ))


def is_hex_address(value):
    return is_string(value) and re.match(r'^(0x)?[a-fA-F0-9]{40}$', force_text(value))


def is_hex_transaction_hash(value):
    return is_string(value) and re.match(r'^(0x)?[a-fA-F0-9]{64}$', force_text(value))



import itertools

from toposort import toposort

from .deferred import (
    generate_registrar_value_setters,
    Bool,
)


class Migration(object):
    migration_id = None
    dependencies = None
    operations = None
    compiled_contracts = None

    def __init__(self, chain):
        self.chain = chain

    @property
    def registrar(self):
        return self.chain.registrar

    @property
    def web3(self):
        return self.chain.web3

    @property
    def migration_key(self):
        if self.migration_id is None:
            raise ValueError("Migrations must have a `migration_id`")
        key = "migration/{migration_id}".format(
            migration_id=self.migration_id,
        )
        return key

    @property
    def has_been_executed(self):
        if not self.registrar.call().exists(self.migration_key):
            return False
        return self.registrar.call().getBool(self.migration_key)

    def mark_as_executed(self):
        Bool(self.chain, key=self.migration_key, value=True).set()

    def process_operation_receipt(self, operation_key, receipt):
        registrar_setters = generate_registrar_value_setters(
            receipt,
            prefix=operation_key,
        )
        for Setter in registrar_setters:
            print("Key: ", Setter.key)
            Setter(self.chain).set()

        # mark the operation as having been completed.
        Bool(self.chain, key=operation_key, value=True).set()

    def execute(self):
        if self.registrar.call().exists(self.migration_key):
            raise ValueError("This migration has already been run")

        for operation_index, operation in enumerate(self.operations):
            operation_key = "{prefix}/operation/{operation_index}".format(
                prefix=self.migration_key,
                operation_index=operation_index,
            )

            operation_alread_executed = (
                self.registrar.call().exists(operation_key) and
                self.registrar.call().getBool(operation_key)
            )
            if operation_alread_executed:
                # raise or continue?
                raise ValueError("This operation has already been run")

            operation_receipt = operation.execute(
                chain=self.chain,
                compiled_contracts=self.compiled_contracts,
            )

            self.process_operation_receipt(operation_key, operation_receipt)

        self.mark_as_executed()


def sort_migrations(migration_classes, flatten=False):
    migration_dependency_graph = {
        m.migration_id: set() if m.dependencies is None else set(m.dependencies)
        for m in migration_classes
    }

    migration_classes_by_id = {
        m.migration_id: m for m in migration_classes
    }
    migration_id_order = toposort(migration_dependency_graph)
    migration_execution_sets = [
        {migration_classes_by_id[migration_id] for migration_id in execution_set}
        for execution_set in migration_id_order
    ]
    if flatten:
        return list(itertools.chain.from_iterable(migration_execution_sets))
    else:
        return migration_execution_sets


def get_migration_classes_for_execution(migration_classes, chain):
    migration_class_execution_sets = sort_migrations(migration_classes)
    migration_execution_sets = [
        {migration_class(chain) for migration_class in execution_set}
        for execution_set
        in migration_class_execution_sets
    ]

    # For each set of migrations in the sorted order, mark whether all of the
    # migrations in that set have been executed.
    execution_set_statuses = [
        all(m.has_been_executed for m in execution_set)
        for execution_set
        in migration_execution_sets
    ]

    # Now find the first set that has not been fully executed and exclude all
    # of the previous sets which have been fully executed.
    try:
        first_executable_set_idx = execution_set_statuses.index(False)
    except ValueError:
        # All migrations have been executed
        return []

    execution_set_candidates = migration_execution_sets[first_executable_set_idx:]

    # Now we check for an invalid state.  If any migration in any migration set
    # past the first unexecuted set has been executed then something is wrong.
    migrations_that_should_not_be_executed = itertools.chain.from_iterable(
        execution_set_candidates[1:]
    )

    if any(m.has_been_executed for m in migrations_that_should_not_be_executed):
        raise ValueError(
            "Something is wrong.  Migrations that depend on un-executed "
            "migrations have already been executed."
        )

    # Get a flattened list of all of the migration instances that still need to
    # be executed.
    migrations_to_run = [
        migration_instance
        for migration_instance
        in itertools.chain.from_iterable(execution_set_candidates)
        if not migration_instance.has_been_executed
    ]

    return migrations_to_run



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os

DIR = os.path.dirname(os.path.abspath(__file__))

try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup, find_packages


readme = open(os.path.join(DIR, 'README.md')).read()


setup(
    name='populus',
    version="1.0.0rc1",
    description="""Ethereum Development Framework""",
    long_description=readme,
    author='Piper Merriam',
    author_email='pipermerriam@gmail.com',
    url='https://github.com/pipermerriam/populus',
    include_package_data=True,
    py_modules=['populus'],
    install_requires=[
        "click>=6.6",
        "configparser==3.5.0",
        "contextlib2>=0.5.4",
        "eth-testrpc>=0.8.0",
        "ethereum-tester-client>=1.1.0",
        "gevent>=1.1.2",
        "py-geth>=1.1.0",
        "py-solc>=0.4.0",
        "pysha3>=0.3",
        "pytest>=2.7.2",
        "toposort>=1.4",
        "web3>=2.1.1",
    ],
    license="MIT",
    zip_safe=False,
    entry_points={
        'console_scripts': ["populus=populus.cli:main"],
        'pytest11': ['ethereum=populus.plugin'],
    },
    keywords='ethereum pytest',
    packages=find_packages(exclude=["tests", "tests.*"]),
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ],
)



import os
import sys
import datetime

from .filesystem import (
    get_blockchains_dir,
)


def get_data_dir(project_dir, chain_name):
    blockchains_dir = get_blockchains_dir(project_dir)
    return os.path.join(blockchains_dir, chain_name)


CHAINDATA_DIR = './chaindata'


def get_chaindata_dir(data_dir):
    return os.path.join(data_dir, CHAINDATA_DIR)


DAPP_DIR = './dapp'


def get_dapp_dir(data_dir):
    return os.path.join(data_dir, DAPP_DIR)


NODEKEY_FILENAME = 'nodekey'


def get_nodekey_path(data_dir):
    return os.path.join(data_dir, NODEKEY_FILENAME)


IPC_FILENAME = 'geth.ipc'


def get_geth_ipc_path(data_dir):
    return os.path.join(data_dir, IPC_FILENAME)


def get_default_datadir_path(testnet=False):
    if testnet:
        testnet = "testnet"
    else:
        testnet = ""

    if sys.platform == 'darwin':
        return os.path.expanduser(os.path.join(
            "~",
            "Library",
            "Ethereum",
            testnet,
        ))
    elif sys.platform.startswith('linux'):
        return os.path.expanduser(os.path.join(
            "~",
            "ethereum",
            testnet,
        ))
    elif sys.platform == 'win32':
        return os.path.expanduser(os.path.join(
            "~",
            "AppData",
            "Roaming",
            "Ethereum",
        ))
    else:
        raise ValueError(
            "Unsupported platform '{0}'.  Only darwin/linux2/win32 are "
            "supported.".format(sys.platform)
        )


def get_default_ipc_path(testnet=False):
    data_dir = get_default_datadir_path(testnet=testnet)

    if sys.platform == 'darwin' or sys.platform.startswith('linux'):
        return os.path.join(data_dir, "geth.ipc")
    elif sys.platform == 'win32':
        return os.path.expanduser(os.path.join(
            "~",
            "AppData",
            "Roaming",
            "Ethereum",
        ))
    else:
        raise ValueError(
            "Unsupported platform '{0}'.  Only darwin/linux2/win32 are "
            "supported.".format(sys.platform)
        )


def get_geth_logfile_path(project_dir, prefix, suffix):
    logs_dir = os.path.join(project_dir, 'logs')
    logfile_name = datetime.datetime.now().strftime(
        'geth-%Y%m%d-%H%M%S-{prefix}-{suffix}.log'.format(
            prefix=prefix, suffix=suffix,
        ),
    )
    return os.path.join(logs_dir, logfile_name)



import functools


def noop(*args, **kwargs):
    pass


def identity(value):
    return value


def combine(f, g):
    return lambda x: f(g(x))


def compose(*functions):
    return functools.reduce(combine, functions, identity)


def apply_formatters_to_return(*formatters):
    formatter = compose(*formatters)

    def outer(fn):
        @functools.wraps(fn)
        def inner(*args, **kwargs):
            value = fn(*args, **kwargs)
            return formatter(value)
        return inner
    return outer


class cached_property(object):
    """
    Source: Django (django.utils.functional)

    Decorator that converts a method with a single self argument into a
    property cached on the instance.

    Optional ``name`` argument allows you to make cached properties of other
    methods. (e.g.  url = cached_property(get_absolute_url, name='url') )
    """
    def __init__(self, func, name=None):
        self.func = func
        self.__doc__ = getattr(func, '__doc__')
        self.name = name or func.__name__

    def __get__(self, instance, type=None):
        if instance is None:
            return self
        res = instance.__dict__[self.name] = self.func(instance)
        return res
